#!/bin/sh
# Copyright (c) 2002-2017 by Dell Inc.
#
# All rights reserved.  This software may not be copied, disclosed,
# transferred, or used except in accordance with a license granted
# by Dell Inc.  This software embodies proprietary information
# and trade secrets of Dell Inc.
#
progress_msg()
{
PROGRESS=$((${PROGRESS}+$1))
if [ -n "${GUI_MODE}" ]; then
echo "Progress: ${PROGRESS}" >&2
log_msg "Progress: ${PROGRESS}"
fi
}
info_msg()
{
log_msg "$*"
if [ -z "${GUI_MODE}" ]; then
echo "`date '+%H:%M:%S'` $*" >&2
fi
}
remove_log_file()
{
if [ -f ${LOGFILE} ]; then
rm -f ${LOGFILE}
fi
}
purge_log_files()
{
for fn in `ls -t /mgtdb/update*.log 2> /dev/null | tail +6`; do
rm -f $fn
done
}
log_msg()
{
if [ ${LOGFILE:-unset} != "unset" ]; then
savmsg="$*"
if [ "${savmsg#[0-9]*:}" != "${savmsg}" ]; then
msg="${savmsg#[0-9]*:}"
savmsg=`echo "${msg}" | awk 'BEGIN {FS = "~"} ; {print $1}'`
fi
echo "`date '+%Y.%m.%d %H:%M:%S'` ${CM_TYPE}: ${savmsg}" >> ${LOGFILE}
if [ -z "${GUI_MODE}" -a -n "${LOG_ALL}" ]; then
echo "`date '+%H:%M:%S'` ${savmsg}" >&2
fi
fi
}
notice_msg()
{
if [ ${NOTICE_FILE:-unset} != "unset" ]; then
echo "$*" >> ${NOTICE_FILE}
fi
}
cmp_version()
{
sv1="${1##V}"
sv2="${2##V}"
ret=$(echo "${sv1}.${sv2}" | awk -F . '
function cmp(a,b) {if (a < b) {return -1} else {if (a > b) {return 1} else {return 0}}}
{if (cmp($1,$4) != 0) {print cmp($1,$4); exit}
else {if (cmp($2,$5) != 0) {print cmp($2,$5); exit}
else {print cmp($3,$6)}}}')
}
mib_get()
{
curdir=$PWD;
if [ ! -d /mgtdb/update/mibwalk ]; then
mkdir -p /mgtdb/update/mibwalk
cd /mgtdb/update/mibwalk
gzip -dc /usr/share/snmp/snmp.tgz 2>/dev/null | tar xf -
fi
cd /mgtdb/update/mibwalk
MIBVAL=`LD_LIBRARY_PATH=/usr/lib:/mgtdb/update/mibwalk/lib /mgtdb/update/mibwalk/bin/snmpwalk -c public -Oqv -M mibs -m all -t 33 localhost $* 2>/dev/null`
if [ "${MIBVAL}" = "No Such Object available on this agent at this OID" ]; then
MIBVAL=
return 1
fi
cd ${curdir}
return 0
}
mib_get_appl_indices()
{
curdir=$PWD;
if [ ! -d /mgtdb/update/mibwalk ]; then
mkdir -p /mgtdb/update/mibwalk
cd /mgtdb/update/mibwalk
gzip -dc /usr/share/snmp/snmp.tgz 2>/dev/null | tar xf -
fi
cd /mgtdb/update/mibwalk
MIBVAL=`LD_LIBRARY_PATH=/usr/lib:/mgtdb/update/mibwalk/lib /mgtdb/update/mibwalk/bin/snmpwalk -c public -Oqs -M /mgtdb/update/mibwalk/mibs -m all -t 33 localhost eqlApplianceState 2>/dev/null | awk '{print $1}' | awk -F . '{print $2}'`
if [ "${MIBVAL}" = "No Such Object available on this agent at this OID" ]; then
MIBVAL=
return 1
fi
cd ${curdir}
return 0
}
version_get()
{
VERSION=
mib_get "eqlControllerMajorVersion.1.$1.$2"
if [ -z "${MIBVAL}" ]; then
return
fi
majver="${MIBVAL}"
mib_get "eqlControllerMinorVersion.1.$1.$2"
minver="${MIBVAL}"
mib_get "eqlControllerMaintenanceVersion.1.$1.$2"
VERSION="${majver}.${minver}.${MIBVAL}"
}
cleanup()
{
rm -f /mgtdb/update/*.gz /mgtdb/update/build.c /mgtdb/update/update.sh /mgtdb/update/ver
rm -f /mgtdb/update/cf* /mgtdb/update/cksum* /mgtdb/update/sec.origconfig
rm -rf /mgtdb/update/mibwalk /mgtdb/update/cemi*.bin /tmp/update.sh /mgtdb/update/sumo*.bin
rm -f /mgtdb/update/ccom*.bin /mgtdb/update/lcem*.bin /mgtdb/update/lsum*.bin /mgtdb/update/*.cem
rm -f /mgtdb/update/sep*.bin
rm -f /mgtdb/update/csep*.bin
rm -f /mgtdb/update/ssep*.bin
rm -f /mgtdb/update/ssem*.bin
rm -f /mgtdb/update/xsep*.bin
rm -rf /mgtdb/update/mibwalk
rm -rf /mgtdb/update/extendcf
rm -rf /mgtdb/*.tmp
rm -f /mgtdb/update/eqlres.jar /mgtdb/update/equall.jar /mgtdb/update/b21523.sh
rm -f /mgtdb/update/patchrom /mgtdb/update/patch.rc /mgtdb/update/vmtck
rm -f /mgtdb/update/patch.inv /mgtdb/update/patch.sh /mgtdb/update/patchrom.asc
rm -f /mgtdb/update/i18n*.rom /mgtdb/update/chalnge
if [ -z "$1" ]; then
if [ -n "${LOCKFILE}" ]; then
rm -f ${LOCKFILE}
fi
fi
}
cleanup_and_exit()
{
log_msg "${CM_TYPE} CM cleaning up and exiting"
log_msg "vmstat -s | awk /times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}"
vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}" \
>>${LOGFILE}
cleanup
exit 88
}
uabort()
{
savmsg=$1
if [ "${1#[0-9]*:}" != "${1}" ]; then
msg="${1#[0-9]*:}"
shift 1
if [ $# -gt 0 ]; then
savcnt=$#
savarg1="$1"
savarg2="$2"
eval set -- \"${msg}\"
${LOGEVENT} 30.4.1 "${shortver1}" "${shortver2}${pl_id}" "$1"
if [ ${GUI_MODE:-unset} = "unset" ]; then
info_msg "${1}."
else
set -- "$savarg1" "$savarg2"
eval set -- \"${savmsg}\"
savmsg=$1
savmsg="${savmsg}~${savarg1}"
if [ ${savcnt} -ge 2 ]; then
savmsg="${savmsg}~${savarg2}"
fi
unset GUI_MODE
info_msg "${savmsg}."
fi
else
${LOGEVENT} 30.4.1 "${shortver1}" "${shortver2}${pl_id}" "$msg"
if [ ${GUI_MODE:-unset} = "unset" ]; then
info_msg "${msg}."
else
unset GUI_MODE
info_msg "${savmsg}."
fi
fi
else
unset GUI_MODE
info_msg "${*}."
${LOGEVENT} 30.4.1 "${shortver1}" "${shortver2}${pl_id}" "$*"
fi
if [ ${CAPTURE_STATE:-unset} = "unset" ]; then
CAPTURE_STATE="Y"
if [ -n "$TwoCMs" -a "${CM_TYPE}" = "active" ]; then
tunnel NOCHECK "test -s /tmp/secondary.update.log"
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ ${ret} -eq 0 ]; then
log_msg "*** Secondary Update log ***"
tunnel "cat /tmp/secondary.update.log" >>${LOGFILE} 2>&1
tunnel "rm -f  /tmp/secondary.update.log"
fi
fi
fi
if [ ${MOVE_UPDATE_LOG:-unset} = "Y" ]; then
if [ -n "$TwoCMs" ]; then
MOVE_UPDATE_LOG=""
DEFLOGFILE="/mgtdb/update`date '+%Y%m%d%H%M%S'`.log"
cat ${LOGFILE} 2>/dev/null | tunnel INPUT "cat >${DEFLOGFILE}"
fi
fi
cleanup_and_exit
}
interlock()
{
UUID0=`dd if=/dev/random count=1 bs=32 2>/dev/null |cat -v`
UUID=`echo $UUID0`
if [ -e /tmp/update.lock ]; then
unset GUI_MODE
info_msg "Another copy of update is already running"
${LOGEVENT} 30.4.1 "${shortver1}" "${shortver2}${pl_id}" "Another copy of update is already running"
exit 88
fi
LOCKFILE="/tmp/update.lock"
echo $UUID > /tmp/update.lock
sleep 1
UUID2=`cat /tmp/update.lock`
if [ "${UUID}" != "${UUID2}" ]; then
unset GUI_MODE
info_msg "Another copy of update is already running"
${LOGEVENT} 30.4.1 "${shortver1}" "${shortver2}${pl_id}" "Another copy of update is already running"
exit 88
fi
}
checkRAentries ()
{
set -- `cachetool | awk 'BEGIN { MATCHEDLINECOUNT = 0; UNMATCHEDLINECOUNT=0 }
/^ *RA entries/ { MATCHEDLINECOUNT++; NUMRAENTRIES = $4; gsub(",","",NUMRAENTRIES); MAXRAENTRIES = $5; gsub(",","",MAXRAENTRIES) }
{ UNMATCHEDLINECOUNT++ }
END { if (UNMATCHEDLINECOUNT < 15) { print UNMATCHEDLINECOUNT; exit } if (MATCHEDLINECOUNT == 0) { exit }
if (length(NUMRAENTRIES) == 0) { print "-1"; exit }
if (length(MAXRAENTRIES) == 0) { print "-2"; exit }
if (MATCHEDLINECOUNT == 1)
{ if ((match(NUMRAENTRIES,"[^0-9]") != 0) || (match(MAXRAENTRIES,"[^0-9]") != 0))
{ print "-3 " NUMRAENTRIES " " MAXRAENTRIES; exit } print NUMRAENTRIES " " MAXRAENTRIES; exit } }
'` > /dev/null 2>&1
RETURNCODE=0
HYBRIDARRAY=""
if test $# -ne 0 && test $# -ne 2; then
log_msg "Unexpected $# RA entry counts ($*) in cachetool output."
RETURNCODE=1
elif test $# -eq 0; then
log_msg "Unaccelerated array OK for update."
elif test $1 -lt 0 || test $1 -gt 200000 || test $2 -lt 0 || (test $2 -gt 300000 && test $1 -ne 0); then
log_msg "Accelerated array with $1/$2 entries; need support for update."
RETURNCODE=2
HYBRIDARRAY="Y"
else
log_msg "Accelerated array with $1/$2 entries OK for update."
HYBRIDARRAY="Y"
fi
return $RETURNCODE
}
KILLED_BY_SIGTERM=143
retryable()
{
if [ $# -ne 2 ]; then
uabort "in 'retryable': wrong # of args; got $# expected 2; args: $*"
fi
log_msg "retryable: $2 ===> $1"
lretry_cnt=0
while [ ${lretry_cnt} -lt 10 ];
do
eval $1
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ ${ret} -eq ${KILLED_BY_SIGTERM} ]; then
log_msg "retryable: ${lretry_cnt}: RETRYING $*"
lretry_cnt=$((${lretry_cnt}+1))
else
if [ ${ret} -ne 0 ]; then
log_msg "retryable: non-zero return code of ${ret}"
log_msg "retryable: ${lretry_cnt}: RETRYING $*"
lretry_cnt=$((${lretry_cnt}+1))
continue
fi
return
fi
done
uabort "1:in 'retryable': \${1} failed to execute successfully; please run update again" "$*"
}
tunnel()
{
if [ -z "$TwoCMs" ]; then
info_msg "JPM: tunnel called on single cm machine"
return
fi
ignore_stdin="-O"
if [ "$1" = "INPUT" ]; then
ignore_stdin=""
shift 1
fi
check_status="yes"
if [ "$1" = "NOCHECK" ]; then
check_status="no"
shift 1
fi
log_msg "tunnel: ipcsh -s ${ignore_stdin} $*"
ipcsh -s ${ignore_stdin} "$*" 2> /tmp/tun.err
status=$?
if [ "${check_status}" = "yes" ]; then
if [ -s /tmp/tun.err ]; then
log_msg "stderr from ipcsh: $(cat /tmp/tun.err)"
fi
if [ $status -ge 95 -a $status -le 99 ]; then
if [ ${MOVE_UPDATE_LOG:-unset} = "Y" ]; then
log_msg "Inter-controller communications failure - $status"
else
uabort "60:Inter-controller communications failure - \${1}" "$status"
fi
fi
fi
rm -rf /tmp/tun.err
return $status
}
countof()
{
if [ -n "$2" ]; then
n=$(tunnel NOCHECK "ps auxw | awk /$1/ | awk 'BEGIN{n=0} !/awk/{n=n+1;} END{print n}'")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
else
n=$(ps auxw | awk /$1/ | awk 'BEGIN{n=0} !/awk/{n=n+1;} END{print n}')
fi
return $n
}
extract_file()
{
ef_kit_file=$1
ef_extract_file=$2
ef_to_dir=$3
curdir=$PWD;
cd ${ef_to_dir}
log_msg "Extracting ${ef_extract_file} from ${ef_kit_file} to ${ef_to_dir}"
gzip -dc ${ef_kit_file} 2>/dev/null | tar xf - "${ef_extract_file}" > /tmp/tar.err 2>&1
ret=$?
cd ${curdir}
if [ ${ret} -ne 0 -o -s /tmp/tar.err ]; then
if [ -s /tmp/tar.err ]; then
log_msg "extract_file: $(cat /tmp/tar.err)"
fi
if [ ${ret} -ne 0 ]; then
log_msg "extract_file: return status ${ret}"
fi
return 1
fi
return 0
}
extract_file_and_check()
{
efc_kit_file=$1
efc_extract_file=$2
efc_to_dir=$3
if [ ! -e /mgtdb/update/cksums ]; then
extract_file "${efc_kit_file}" cksums /mgtdb/update
fi
set -- $(awk "/ ${efc_extract_file}$/ {print}" /mgtdb/update/cksums)
efc_cksum=$1
efc_size=$2
extract_file "${efc_kit_file}" "${efc_extract_file}" ${efc_to_dir}
ret=$?
if [ ${ret} -ne 0 ]; then
return 1
fi
set -- $(cksum ${efc_to_dir}/"${efc_extract_file}")
if [ "${efc_cksum}" = "$1" -a "${efc_size}" = "$2" ]; then
return 0
fi
log_msg "checksum error in ${efc_extract_file}, $1 $2 instead of ${efc_cksum} ${efc_size}"
return 1
}
push2secondary()
{
if [ $# -ne 2 ]; then
uabort "in 'push2secondary': 2 args required, received only $#"
fi
log_msg "push2secondary: $*"
tunnel "echo secondary:;mount;df;vmstat -s | awk \"/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}\" " >> ${LOGFILE}
p2s_inputfile=$1
p2s_outputfile=$2
p2s_retry_cnt=0
msize=0
pswitch=""
if [ -n "${GUI_MODE}" ]; then
msize=$(ls -l $1 | awk '{print $5}')
msize=$((($msize+1048575)/1048576))
pswitch="-p ${PROGRESS}"
fi
p2indir="${p2s_inputfile#/}"
if [ "${p2indir}" = "${p2s_inputfile}" ]; then
p2indir="."
else
p2indir="/${p2indir%/*}"
fi
p2dir1="${p2s_outputfile#/}"
p2dir1="/${p2dir1%%/*}"
while [ ${p2s_retry_cnt} -lt 10 ];
do
cat ${p2s_inputfile} 2>/dev/null | ipcsh -s ${pswitch} cat \> ${p2s_outputfile}
if [ "${p2dir1}" = "${CFRoot}" ]; then
tunnel "${UPDATE_SH2} ChangeCFCardStatus ASYNC"
fi
sec_cksum=$(tunnel "cat ${p2s_outputfile} | cksum")
if [ -f "${p2indir}/cksums" ]; then
orig_cksum=$(awk "/ ${p2s_inputfile##*/}$/" ${p2indir}/cksums | awk '{print $1 " " $2}')
else
orig_cksum=""
fi
if [ -z "${orig_cksum}" ]; then
here=$PWD;
if [ "${p2s_inputfile%/*}" != "${p2s_inputfile}" ]; then cd ${p2s_inputfile%/*}; fi
orig_cksum=$(cat ${p2s_inputfile##*/} | cksum)
cd ${here}
fi
if [ "${orig_cksum}" != "${sec_cksum}" ]; then
log_msg "push2secondary: ${p2s_retry_cnt}: RETRYING cksum mismatch on ${p2s_inputfile}:"
log_msg "    orig: ${orig_cksum}"
log_msg "    sec:  ${sec_cksum}"
p2s_retry_cnt=$((${p2s_retry_cnt}+1))
continue
else
if [ -n "${GUI_MODE}" ]; then
PROGRESS=$((${PROGRESS}+((${msize}-1)*5)))
fi
return
fi
done
uabort "3:in 'push2secondary': Reached retry limit of \${1} for cmd: \${2}" "${p2s_retry_cnt}" "$*"
}
extract2local()
{
if [ $# -ne 2 ]; then
uabort "in 'extract2local': 2 args required, received only $#"
fi
log_msg "extract2local: $*"
(mount;df) >> ${LOGFILE}
log_msg "vmstat -s | awk /times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}"
vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}" \
>>${LOGFILE}
retry_cnt=0
msize=0
pswitch=""
source_file=$1
dest_file=$2
if [ -n "${GUI_MODE}" ]; then
msize=`cat ${WDir}/cksums | awk "/ ${source_file}$/" | awk '{print int(($2+1048575)/1048576)}'`
pswitch="-p ${PROGRESS}"
fi
while [ ${retry_cnt} -lt 10 ];
do
gzip -dc ${WDir}/${GZFile} 2>/dev/null | tar xOf - "${source_file}" 2>/tmp/tar.err | ipcsh ${pswitch} 32766 cat \> ${dest_file}
if [ -s /tmp/tar.err ]; then
log_msg "extract2local: $(cat /tmp/tar.err)"
retry_cnt=$((${retry_cnt}+1))
continue
fi
sync;sync;sync
orig_cksum=$(awk "/ ${1##*/}$/" /mgtdb/update/cksums)
here=$PWD
cd
ChangeCFCardStatus ASYNC
cd ${2%/*}
new_cksum=$(cksum ${2##*/})
cd ${here}
if [ "${orig_cksum}" != "${new_cksum}" ]; then
log_msg "extract2local: ${retry_cnt}: RETRYING cksum mismatch:  $*"
log_msg "    orig: ${orig_cksum}"
log_msg "    new:  ${new_cksum}"
retry_cnt=$((${retry_cnt}+1))
continue
else
if [ -n "${GUI_MODE}" ]; then
PROGRESS=$((${PROGRESS}+((${msize}-1)*5)))
fi
return
fi
done
uabort "67:in 'extract2local': Reached retry limit of \${1} for cmd: \${2}" "${retry_cnt}" "$*"
}
push2local()
{
if [ $# -ne 2 ]; then
uabort "in 'push2local': 2 args required, received only $#"
fi
log_msg "push2local: $*"
(mount;df) >> ${LOGFILE}
log_msg "vmstat -s | awk /times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}"
vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}" \
>>${LOGFILE}
retry_cnt=0
while [ ${retry_cnt} -lt 10 ];
do
cp -f $1 $2 2>/dev/null
ret=$?
if [ ${ret} -ne 0 ]; then
log_msg "push2local: ${retry_cnt}: RETRYING cp failure:  $*"
retry_cnt=$((${retry_cnt}+1))
continue
fi
sync;sync;sync
if [ -f /mgtdb/update/cksums ]; then
orig_cksum=$(awk "/ ${1##*/}$/" /mgtdb/update/cksums)
else
orig_cksum=""
fi
if [ -z "${orig_cksum}" ]; then
here=$PWD;
if [ "${1%/*}" != "$1" ]; then cd ${1%/*}; fi
orig_cksum=$(cksum ${1##*/})
cd ${here}
fi
here=$PWD
cd
ChangeCFCardStatus ASYNC
cd ${2%/*}
new_cksum=$(cksum ${2##*/})
cd ${here}
if [ "${orig_cksum}" != "${new_cksum}" ]; then
log_msg "push2local: ${retry_cnt}: RETRYING cksum mismatch:  $*"
log_msg "    orig: ${orig_cksum}"
log_msg "    new:  ${new_cksum}"
retry_cnt=$((${retry_cnt}+1))
continue
else
return
fi
done
uabort "4:in 'push2local': Reached retry limit of \${1} for cmd: \${2}" "${retry_cnt}" "$*"
}
update_config()
{
BP="primary"
if [ -n "$1" ]; then
BP="backup"
fi
CR=$(awk 'BEGIN {printf "\r"; exit}')
retryable 'sed -e "s/^BOOTPATH=.*/BOOTPATH=${BP}${CR}/"
-e "s/^CPU0_FILE=.*/CPU0_FILE=eqlstor.gz${CR}/"
-e "s/^CPU1_FILE=.*/CPU1_FILE=eqlqrq.gz${CR}/"
${CFRoot}/config.txt > /tmp/config.txt' "point at ${BP} dir and our images"
retryable 'oldcs=$(cd ${CFRoot}; cksum config.txt 2>/dev/null)' "getting checksum of ${CFRoot}/config.txt"
retryable 'newcs=$(cd /tmp; cksum config.txt 2>/dev/null)' "getting checksum of /tmp/config.txt"
if [ "${oldcs}" != "${newcs}" ]; then
log_msg "Changing config.txt BOOTPATH to ${BP}"
cp -f ${CFRoot}/config.txt ${CFRoot}/config.bak
cp -f /tmp/config.txt ${CFRoot}/config.txt
fi
rm /tmp/config.txt
}
copy_directory()
{
if [ $# -ne 2 ]; then
uabort "in 'copy_directory': wrong # of args; got $# expected 2; args: $*"
fi
log_msg "copy_directory: $1 ===> $2"
cd $1
for afile in *;
do
if [ ${afile} = "*" ]; then
break
fi
if [ ! -d ${afile} ]; then
info_msg "Copying ${afile}"
push2secondary ${afile} $2/${afile}
else
info_msg "Copying directory ${afile}"
cd ${afile}
retryable "tunnel \"mkdir -p $2/${afile}\"" "create directory ${afile}"
copy_directory $1/${afile} $2/${afile}
cd $1
fi
done
}
common_initialization()
{
trap cleanup_and_exit 2 3 9
UPDATE_SH=$0
UPDATE_VERSION=100
UPDATE_SH2=/sbin/update.sh
WDir="/mgtdb/update"
CFRoot="/pss"
NDRHook="${CFRoot}/update/ndr.sh"
NDRHookTmp="/tmp/ndr.sh"
NDRDowngradeInProgress=${CFRoot}/update/d_update.flg
NDREMailFile=/mgtdb/NDREmailrunonce.sh
NDRUseRunningUpdate=${CFRoot}/update/r_update.flg
LOGEVENT=true
if [ -x /sbin/logevent ]; then
LOGEVENT=/sbin/logevent
fi
running_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /build.c)
if [ -r /var/patch/build.c ] ; then
running_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /var/patch/build.c)
fi
shortver1="${running_ver##* V}"
shortver1="V${shortver1%% *}"
B2BStat=$(cord -b)
case "${B2BStat}" in
*Active?CM*)
CM_TYPE="active"
DEFLOGFILE="/mgtdb/update`date '+%Y%m%d%H%M%S'`.log"
LOGFILE=${LOGFILE:-${DEFLOGFILE}}
;;
*)
CM_TYPE="secondary"
LOGFILE=/tmp/secondary.update.log
;;
esac
set -- `echo ${B2BStat} | sed -e 's/^.*)://' -e 's/\..*$//' | awk '{Reg=substr($3,4,1); print "CM" (Reg % 2) " CM" (++Reg % 2)}'`
ThisCM="$1"
case "${B2BStat}" in
*channel?Down*)
TwoCMs=""
;;
*channel?Up*)
TwoCMs="Yes"
;;
*)
uabort "5:Unknown number of control modules"
;;
esac
lretry_cnt=0
while [ ${lretry_cnt} -lt 10 ]
do
cmtype=$(ecli <<END | awk '/Model:/{print $2; exit 0}')
cm
END
cmtype2=$(ecli <<END | awk '/Controller Type:/{print $4; exit 0}')
cm
END
if [ -z "$cmtype" ]
then
sleep 10
lretry_cnt=$((${lretry_cnt}+1))
else
break
fi
done
lretry_cnt=0
while [ ${lretry_cnt} -lt 10 ]
do
platform=$(ecli <<END | awk '/Executing in/{print $5; exit 0}')
cmv
END
if [ -z "$platform" ]
then
sleep 10
lretry_cnt=$((${lretry_cnt}+1))
else
break
fi
done
CEMI_TYPE=""
CEMI2_TYPE=""
UCEMI_TYPE=""
SEP_TYPE=""
SEM_TYPE=""
if cm_isa_lh; then
UCEMI_TYPE=".cem"
elif enc_isa_sumo; then
CEMI_TYPE="sumo"
CEMI2_TYPE="cemi"
elif enc_isa_m3; then
CEMI_TYPE="ccom"
elif enc_isa_1603; then
CEMI_TYPE="cemi"
CEMI2_TYPE="sumo"
elif cm_isa_shogun; then
SEP_TYPE="ssep"
SEM_TYPE="sem"
elif cm_isa_pirate_xlp; then
SEP_TYPE="xsep"
elif cm_isa_pirate; then
SEP_TYPE="sep"
elif cm_isa_reposado; then
SEP_TYPE="csep"
fi
}
file_matches()
{
fn=$(echo $1 | awk '{print tolower($0)}')
shift 1
for str in $*; do
if [ "${str}" != "${str#.}" ]; then
if [ "${fn}" != "${fn%${str}}" ]; then
return 0
fi
else
if [ "${fn}" != "${fn#${str}}" ]; then
return 0
fi
fi
done
return 1
}
verify_checksums()
{
here=$PWD;
cd $1
if [ -s cksums ]; then
while read aline ; do
retryable 'fname=$(echo ${aline} 2>/dev/null | awk "{print \$3}" 2>/dev/null )' "getting filenames"
if file_matches "${fname}" eqlcfe.gz eqlx cemi sumo ccom lcem lsum .bin .cem ipcsh extendcf; then
if file_matches "${fname}" "${CEMI_TYPE}" ${CEMI2_TYPE} ${UCEMI_TYPE} ${SEP_TYPE} ${SEM_TYPE}; then
check="yes"
else
check="no"
fi
else
check="yes"
fi
if file_matches "${fname}" ipsec.rom; then
if cm_supports_ipsec; then
check="yes"
else
check="no"
fi
fi
if [ "${check}" = "yes" ]; then
retryable 'result=$(cksum ${fname} 2>/dev/null)' "verifying checksum of ${fname}"
if [ "${aline}" != "${result}" ]; then
log_msg "checksum error in $1, ${result} instead of ${aline}"
cd ${here}
return 1
fi
fi
done < cksums
else
log_msg "cksums file in $1 missing or null"
cd ${here}
return 1
fi
cd ${here}
return 0
}
do_cemi_ecli()
{
sleep 60
/sbin/ecli >/tmp/ecli.log 2>/dev/null <<- "EOF"
ses cli lsidiag install
q
EOF
eclistat=$(awk '/defaults installed OK/{print}' /tmp/ecli.log)
if [ "${CM_TYPE}" = "secondary" ]; then
log_msg "ecli lsidiag install executed, status not checked on secondary"
return
fi
if [ -n "${eclistat}" ]; then
log_msg "ecli lsidiag install succeeded"
return
else
log_msg "Retrying ecli lsidiag install"
lretry_cnt=1
while [ ${lretry_cnt} -lt 11 ];
do
sleep 15
/sbin/ecli >/tmp/ecli.log 2>/dev/null <<- "EOF"
ses cli lsidiag install
q
EOF
eclistat=$(awk '/defaults installed OK/{print}' /tmp/ecli.log)
if [ -n "${eclistat}" ]; then
log_msg "ecli lsidiag install succeeded after ${lretry_cnt} retries"
return
else
log_msg "ecli lsidiag install failed, retry ${lretry_cnt}"
lretry_cnt=$((${lretry_cnt}+1))
fi
done
log_msg "ecli lsidiag install failed"
fi
}
enc_isa_1603()
{
if [ "$platform" = "RS1603" ]
then
return 0
else
return 1
fi
}
enc_isa_sumo()
{
if [ "$platform" = "SUMO" ]
then
return 0
else
return 1
fi
}
enc_isa_m3()
{
if [ "$cmtype" = "70-0115" ]
then
return 0
else
return 1
fi
}
cm_isa_lh()
{
if [ "$cmtype" = "70-0300" ]
then
return 0
else
return 1
fi
}
cm_isa_pirate_xlp()
{
if [ "$cmtype" = "70-0425" ]
then
return 0
elif [ "$cmtype" = "70-0485" ]
then
return 0
else
return 1
fi
}
cm_isa_pirate()
{
if [ "$cmtype" = "70-0400" ]
then
return 0
elif [ "$cmtype" = "70-0476" ]
then
return 0
elif [ "$cmtype" = "70-0477" ]
then
return 0
elif [ "$cmtype" = "70-0478" ]
then
return 0
else
return 1
fi
}
cm_isa_reposado()
{
if [ "$cmtype" = "70-0450" ]
then
return 0
else
return 1
fi
}
cm_isa_shogun()
{
if [ "$platform" = "SHOGUN" ]
then
return 0
else
return 1
fi
}
cm_supports_ipsec()
{
if [ "$cmtype" = "70-0001" ]; then
return 1
elif [ "$cmtype" = "70-0011" ]; then
return 1
elif [ "$cmtype" = "70-0101" ]; then
return 1
elif [ "$cmtype" = "70-0111" ]; then
return 1
elif [ "$cmtype" = "70-0115" ]; then
return 1
elif [ "$cmtype" = "70-0120" ]; then
return 1
else
return 0
fi
}
get_active_slave_sep_version()
{
echo tv | /sbin/ecli | awk '/^Passive CM SEP Version/ { print tolower($6) }'
}
get_active_sep_version()
{
echo tv | /sbin/ecli | awk '/^Active CM SEP Version/ { print tolower($6) }'
}
do_cemi_update()
{
if cm_isa_lh
then
log_msg "Upgrading LH CEMI to ${1}"
/sbin/cemi_update -f "$1"
return 0
fi
if enc_isa_sumo
then
doecli="no"
elif enc_isa_m3
then
doecli="no"
elif enc_isa_1603
then
doecli="yes"
elif cm_isa_pirate
then
doecli="no"
elif cm_isa_pirate_xlp
then
doecli="no"
elif cm_isa_reposado
then
doecli="no"
else
return 0
fi
cemi_fn="$1"
if [ -n "${SEP_TYPE}" ]
then
current_cemi_ver=`get_active_slave_sep_version`
new_cemi_ver=${cemi_fn%.bin}
new_cemi_ver=${new_cemi_ver##*/${SEP_TYPE}}
if [ "${current_cemi_ver}" != "${new_cemi_ver}" ]; then
log_msg "Upgrading SEP from ${current_cemi_ver} to ${new_cemi_ver}"
/sbin/sep_update -R -s ${cemi_fn} >>${LOGFILE} 2>&1
ret=$?
if [ ${ret} -eq 1 ]; then
/sbin/sep_update -s ${cemi_fn} >>${LOGFILE} 2>&1
ret=$?
if [ ${ret} -ne 0 ]; then
log_msg "Return status of ${ret}"
fi
fi
return ${ret}
fi
return 0
else
current_cemi_ver=$(cemi_update -v 2>/dev/null)
ret=$?
new_cemi_ver=${cemi_fn%.bin}
new_cemi_ver=${new_cemi_ver##*/${CEMI_TYPE}}
if [ ${ret} -eq 0 ]; then
if [ "${current_cemi_ver}" != "${new_cemi_ver}" ]; then
log_msg "Upgrading CEMI from ${current_cemi_ver} to ${new_cemi_ver}"
/sbin/cemi_update -f ${cemi_fn} >>${LOGFILE} 2>&1
if [ "${doecli}" = "yes" ]; then
do_cemi_ecli
fi
fi
elif [ ${ret} -ne 6 ]; then
info_msg "Error getting CEMI firmware version; CEMI update skipped"
return ${ret}
fi
return 0
fi
}
get_secondary_ver()
{
secondary_ver=$(tunnel "awk '/EqualLogic_Version/{sub(\".* \x22\", \"\");sub(\"\x22;\", \"\" );print }' /build.c")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
}
force_secondary_version_match()
{
stty -isig
trap '' INT HUP QUIT TERM PIPE
push2secondary ${UPDATE_SH} /tmp/update.sh
tunnel "chmod +x /tmp/update.sh"
UPDATE_SH2=/tmp/update.sh
retryable "BPLine=$(awk '/BOOTPATH/{sub("\r$","");print}' ${CFRoot}/config.txt 2>/dev/null)" "get BPLine"
BPDir=${BPLine#BOOTPATH=}
if [ -z ${BPDir} ]; then
uabort "66:Cannot copy files from a network-booted configuration"
fi
retryable "SecBPLine=$(tunnel cat ${CFRoot}/config.txt | awk '/BOOTPATH/{sub("\r$","");print}')" "get SecBPLine"
SecBPDir=${SecBPLine#BOOTPATH=}
if [ -z ${SecBPDir} ]; then
SecBPDir="network"
fi
info_msg "Updating secondary controller to same firmware revision"
info_msg "as the active controller."
info_msg "This will take several minutes."
log_msg "Stopping dbd on secondary controller"
tunnel "/sbin/eqlinit stop dbd" >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
tunnel "/sbin/eqlinit stop agent" >>${LOGFILE} 2>&1
retryable 'tunnel "${UPDATE_SH2} ChangeCFCardStatus ASYNC"' "mount 2ndary async"
info_msg "Secondary controller is booting from ${SecBPDir}"
if [ ${SecBPDir} = "primary" ]; then
retryable 'tunnel "rm -rf /pss/backup/*;
mv /pss/backup /pss/ndrtmp;
mv /pss/primary /pss/backup;
mv /pss/ndrtmp /pss/primary;
"'          "on 2ndary move files from primary to backup"
retryable 'tunnel "${UPDATE_SH2} update_config B"' "update 2ndary config.txt"
else
retryable 'tunnel "rm -rf /pss/primary/*"' "on 2ndary clear primary"
fi
retryable 'tunnel "${UPDATE_SH2} ChangeCFCardStatus ASYNC"' "flush 2ndary cache"
cd ${CFRoot}
for rfile in *;
do
if [ ! -d ${rfile} \
-a "${rfile}" != "config.txt" \
-a "${rfile}" != "config.bak" ]
then
info_msg "Copying ${rfile}"
push2secondary ${rfile} ${CFRoot}/${rfile}
else
if [ -d ${rfile} ]; then
FoundBPDir=`echo ${rfile} | awk -v BP=${BPDir} '{if (tolower($0) == tolower(BP)) print}'`
if [ ${#FoundBPDir} -gt 0 ]; then
info_msg "Copying directory ${rfile}"
copy_directory ${CFRoot}/${BPDir} ${CFRoot}/primary
cd ${CFRoot}
else
info_msg "Ignoring directory ${rfile}"
fi
fi
fi
done
retryable 'tunnel "${UPDATE_SH2} update_config"' "update 2ndary config.txt"
retryable 'tunnel "${UPDATE_SH2} ChangeCFCardStatus ASYNC"' "flush 2ndary cache"
log_msg "*** Secondary Update log ***"
tunnel "cat /tmp/secondary.update.log" >>${LOGFILE} 2>&1
tunnel "rm -f  /tmp/secondary.update.log"
info_msg "Update complete, rebooting secondary controller."
tunnel NOCHECK "sync;reboot"
info_msg "Secondary controller has been updated and is being rebooted"
stty isig
trap INT HUP QUIT TERM PIPE
loop_cnt=0
snooze_time=10
while :
do
sleep ${snooze_time}
status=$(cord -b | awk '/channel Up/')
if [ -n "${status}" ]; then
sleep 20
lretry_cnt=0
tunnel NOCHECK "test -s /tmp/PrefetchDone"
ret=$?
while [ ${ret} -ne 0 ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 36 ]; then
info_msg "Waiting for secondary control module Prefetch synchronization..."
sleep 5
tunnel NOCHECK "test -s /tmp/PrefetchDone"
ret=$?
else
log_msg "Secondary control module Prefetch taking longer than 3 minutes."
break
fi
done
get_secondary_ver
ret=$?
if [ ${ret} -ne 0 ]; then
if [ ${ret} -eq 88 ]; then exit ; fi
uabort "6:Fatal unexpected error \${1} when trying to verify version of firmware on secondary control module" "${ret}"
fi
if [ "${primary_ver}" = "${secondary_ver}" ]; then
break;
fi
fi
loop_cnt=$(($loop_cnt+1))
if [ ${loop_cnt} -gt 90 ]; then
uabort "7:Secondary control module reboot unsuccessful"
fi
if [ $((${loop_cnt}/12*12)) -eq ${loop_cnt} ]; then
info_msg "Waiting up to $((15-${loop_cnt}/6)) minutes more for secondary control module to reboot"
else
info_msg "Waiting for secondary control module to reboot..."
fi
done
}
update_secondary_cm()
{
if [ -n "${pl_id}" ]; then
retryable "CFDestDir=$(tunnel cat ${CFRoot}/config.txt | awk '/BOOTPATH/{sub("\r$","");print}')" "get CFDestDir"
CFDestDir=${CFDestDir#BOOTPATH=}
if [ -z ${CFDestDir} ]; then
info_msg "Secondary controller booting from network."
info_msg "Unable to update secondary controller."
return
fi
fi
log_msg "Stopping dbd on secondary controller"
tunnel "/sbin/eqlinit stop dbd" >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
tunnel "/sbin/eqlinit stop agent" >>${LOGFILE} 2>&1
log_msg "secondary flash contents at update start:"
tunnel "/bin/ls -alR ${CFRoot}" >>${LOGFILE} 2>&1
tunnel "sync; ${UPDATE_SH2} testFlash"
info_msg "Updating flash memory on secondary controller."
info_msg "This will take several minutes."
if [ "${pl_id# (L}" != "${pl_id}" ]; then
progress_msg 12
cd ${WDir}
tunnel "if [ ! -e ${CFRoot}/${CFDestDir}/patch ]; then mkdir ${CFRoot}/${CFDestDir}/patch; fi"
tunnel "if [ ! -e /var/patch ]; then mkdir /var/patch; fi"
push2secondary ${CFRoot}/patch.rc ${CFRoot}/patch.rc
progress_msg 12
push2secondary ${CFRoot}/${CFDestDir}/patch/patch.sh ${CFRoot}/${CFDestDir}/patch/patch.sh
progress_msg 12
push2secondary ${CFRoot}/${CFDestDir}/patch/patchrom ${CFRoot}/${CFDestDir}/patch/patchrom
progress_msg 12
push2secondary ${CFRoot}/${CFDestDir}/patch/patchrom.asc ${CFRoot}/${CFDestDir}/patch/patchrom.asc
progress_msg 12
push2secondary ${CFRoot}/${CFDestDir}/patch/patch.inv ${CFRoot}/${CFDestDir}/patch/patch.inv
progress_msg 12
if [ -e ${CFRoot}/${CFDestDir}/patch/equall.jar ]; then
push2secondary ${CFRoot}/${CFDestDir}/patch/equall.jar ${CFRoot}/${CFDestDir}/patch/equall.jar
progress_msg 12
fi
if [ -e ${CFRoot}/${CFDestDir}/patch/eqlres.jar ]; then
push2secondary ${CFRoot}/${CFDestDir}/patch/eqlres.jar ${CFRoot}/${CFDestDir}/patch/eqlres.jar
progress_msg 12
fi
if [ -e ${CFRoot}/${CFDestDir}/patch/fxrplm.sh ]; then
push2secondary ${CFRoot}/${CFDestDir}/patch/fxrplm.sh ${CFRoot}/${CFDestDir}/patch/fxrplm.sh
progress_msg 12
fi
if [ -e ${CFRoot}/${CFDestDir}/patch/cemiup.sh ]; then
push2secondary ${CFRoot}/${CFDestDir}/patch/cemiup.sh ${CFRoot}/${CFDestDir}/patch/cemiup.sh
progress_msg 12
fi
if [ -e ${CFRoot}/${CFDestDir}/patch/b21523.sh ]; then
push2secondary ${CFRoot}/${CFDestDir}/patch/b21523.sh ${CFRoot}/${CFDestDir}/patch/b21523.sh
progress_msg 12
fi
else
if [ "${pl_id# (T}" != "${pl_id}" ]; then
progress_msg 12
cd ${WDir}
tunnel "if [ ! -e ${CFRoot}/${CFDestDir}/lang ]; then mkdir ${CFRoot}/${CFDestDir}/lang; fi"
push2secondary ${CFRoot}/${CFDestDir}/lang/i18n-${pl_lang}.rom ${CFRoot}/${CFDestDir}/lang/i18n-${pl_lang}.rom
progress_msg 12
push2secondary ${CFRoot}/${CFDestDir}/lang/build-${pl_lang}.c ${CFRoot}/${CFDestDir}/lang/build-${pl_lang}.c
progress_msg 12
else
retryable 'tunnel "${UPDATE_SH2} prep_for_update ${shortver1} ${shortver2}"' "prep 2ndary for update"
progress_msg 2
CFDestDir=update
cmp_version "${shortver2}" "${shortver1}"
if [ $ret -eq -1 ]; then
tunnel "touch ${NDRUseRunningUpdate}"
tunnel "touch ${NDRDowngradeInProgress}"
info_msg "Create Downdgrade In Progress file on secondary"
fi
cd ${WDir}
push2secondary ${CFRoot}/${CFDestDir}/eqlstor.gz ${CFRoot}/${CFDestDir}/eqlstor.gz
progress_msg 5
push2secondary ${CFRoot}/${CFDestDir}/eqlqrq.gz ${CFRoot}/${CFDestDir}/eqlqrq.gz
progress_msg 6
push2secondary ${BUILD_FILE} ${CFRoot}/${CFDestDir}/${BUILD_FILE}
progress_msg 1
push2secondary cksums ${CFRoot}/${CFDestDir}/cksums
progress_msg 1
if [ -n "${cemi_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${cemi_file} ${CFRoot}/${CFDestDir}/${cemi_file}
progress_msg 3
fi
if [ -n "${cemi2_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${cemi2_file} ${CFRoot}/${CFDestDir}/${cemi2_file}
progress_msg 3
fi
if [ -n "${ucemi_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${ucemi_file} ${CFRoot}/${CFDestDir}/${ucemi_file}
progress_msg 3
fi
if [ -n "${sep_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${sep_file} ${CFRoot}/${CFDestDir}/${sep_file}
progress_msg 3
fi
if [ -n "${sem_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${sem_file} ${CFRoot}/${CFDestDir}/${sem_file}
progress_msg 3
fi
if [ -n "${ipsec_file}" ]; then
push2secondary ${CFRoot}/${CFDestDir}/${ipsec_file} ${CFRoot}/${CFDestDir}/${ipsec_file}
progress_msg 4
fi
if [ -s ${CFRoot}/${CFDestDir}/sernum.bin ]; then
push2secondary ${CFRoot}/${CFDestDir}/sernum.bin ${CFRoot}/${CFDestDir}/sernum.bin
fi
push2secondary update.sh ${CFRoot}/${CFDestDir}/update.sh
tunnel "chmod +x ${CFRoot}/${CFDestDir}/update.sh"
tunnel "if [ \"`ls ${CFRoot}/primary/lang/build-*.c 2>/dev/null`\" = \"\" ]; then exit 1; else exit 0;fi"
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ ${ret} -eq 0 ]; then
tunnel "${UPDATE_SH2} copy_lang_folder ${CFDestDir}"
fi
fi
fi
retryable 'tunnel "sync; ${UPDATE_SH2} testFlash"' "test flash on 2ndary"
progress_msg 2
retryable 'tunnel "${UPDATE_SH2} ChangeCFCardStatus SYNC"' "mount 2ndary sync"
log_msg "secondary flash contents at update end:"
tunnel "/bin/ls -alR ${CFRoot}" >>${LOGFILE} 2>&1
if [ "${pl_id# (L}" != "${pl_id}" ]; then
tunnel "${CFRoot}/${CFDestDir}/patch/patch.sh" >>${LOGFILE} 2>&1
info_msg "Update of flash memory on secondary controller completed."
else
if [ "${pl_id# (T}" != "${pl_id}" ]; then
log_msg "Mounting ${pl_lang} language pack on Secondary"
tunnel "umount /var/web/languages/${pl_lang}" >>/dev/null 2>&1
tunnel "mkdir -p /var/web/languages/${pl_lang}" >>/dev/null 2>&1
tunnel "mount -v -t romfs -o ro ${CFRoot}/${CFDestDir}/lang/i18n-${pl_lang}.rom /var/web/languages/${pl_lang}" >>${LOGFILE} 2>&1
info_msg "Update of flash memory on secondary controller completed."
else
log_msg "cksums:"
tunnel "cat ${CFRoot}/${CFDestDir}/cksums" >>${LOGFILE} 2>&1
log_msg "build.c:"
tunnel "cat ${CFRoot}/${CFDestDir}/build.c" >>${LOGFILE} 2>&1
info_msg "Update of flash memory on secondary controller completed."
fi
fi
log_msg "Starting dbd on secondary controller"
tunnel "/sbin/eqlinit start /sbin/dbd" >>${LOGFILE} 2>&1
log_msg "Starting agent on secondary controller"
tunnel "/sbin/eqlinit start /usr/bin/agent" >>${LOGFILE} 2>&1
log_msg "*** Secondary Update log ***"
tunnel "cat /tmp/secondary.update.log" >>${LOGFILE} 2>&1
tunnel "rm -f  /tmp/secondary.update.log"
}
copy_lang_folder()
{
CFDestDir="$1"
new_major=`awk '/EQL_MAJREV/{print $3}' /${CFRoot}/${CFDestDir}/build.c`
new_minor=`awk '/EQL_MINREV/{print $3}' /${CFRoot}/${CFDestDir}/build.c`
lang_revmin=`awk 'BEGIN {n=0} /LANG_REV_MIN/{{print $3} {n=1}} END { if (n==0) {print n}}' /${CFRoot}/${CFDestDir}/build.c`
lang_revrec=`awk 'BEGIN {n=0} /LANG_REV_REC/{{print $3} {n=1}} END { if (n==0) {print n}}' /${CFRoot}/${CFDestDir}/build.c`
for lang_bldc in `ls ${CFRoot}/primary/lang/build*c 2>/dev/null`; do
if [ "${lang_bldc}." != "." ]; then
lang_major=`awk '/EQL_MAJREV/{print $3}' ${lang_bldc}`
lang_minor=`awk '/EQL_MINREV/{print $3}' ${lang_bldc}`
TmpVer=$(awk '/EqualLogic_Version/{sub(".*= \x22", "");sub(".;$", "" );print}' ${lang_bldc})
lang_level="${TmpVer##* (T}"
if [ "${lang_level}" = "${TmpVer}" ]; then
return
else
lang_level_with_lang="${lang_level%%)*}"
lang_level="${lang_level%%_*}"
lang_name="${lang_level_with_lang##*_}"
fi
if [ "${new_major}" = "${lang_major}" -a "${new_minor}" = "${lang_minor}" -a ${lang_level} -ge ${lang_revmin} ]; then
if [ ${lang_level} -lt ${lang_revrec} ]; then
if [ "${CM_TYPE}" = "active" ]; then
${LOGEVENT} 30.3.3 "V${lang_major}.${lang_minor} (T${lang_level_with_lang})" "V${new_major}.${new_minor} (T${lang_revrec})"
fi
log_msg "The language kit version V${lang_major}.${lang_minor} (T${lang_level_with_lang}) is below the recommended version V${new_major}.${new_minor} (T${lang_revrec})."
fi
log_msg "Copying language kit T${lang_level_with_lang}"
mkdir -p ${CFRoot}/${CFDestDir}/lang/
cp ${lang_bldc} ${CFRoot}/${CFDestDir}/lang/
cp "${CFRoot}/primary/lang/i18n-${lang_name}.rom" ${CFRoot}/${CFDestDir}/lang/
else
if [ "${new_major}" != "${lang_major}" -o "${new_minor}" != "${lang_minor}" ]; then
if [ "${CM_TYPE}" = "active" ]; then
${LOGEVENT} 30.3.2 "V${lang_major}.${lang_minor} (T${lang_level_with_lang})" "V${new_major}.${new_minor}"
fi
log_msg "The language kit version V${lang_major}.${lang_minor} (T${lang_level_with_lang}) does not match firmware version V${new_major}.${new_minor}."
else
if [ "${CM_TYPE}" = "active" ]; then
${LOGEVENT} 30.3.1 "V${lang_major}.${lang_minor} (T${lang_level_with_lang})" "V${new_major}.${new_minor} (T${lang_revmin})"
fi
log_msg "The language kit version V${lang_major}.${lang_minor} (T${lang_level_with_lang}) is below the minimum version V${new_major}.${new_minor} (T${lang_revmin})."
fi
fi
fi
done
}
verify_platform_compat()
{
here=$PWD;
cd ${WDir}
if [ -e build.c ]; then
supp_cms=$(awk '/Supported_Controllers/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
supp_platforms=$(awk '/Supported_Platforms/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
fi
NewVer=$(awk '/EqualLogic_Version/{sub(".*= \x22", "");sub(".;$", "" );print}' \
/mgtdb/update/build.c)
shortver2="${NewVer##* V}"
shortver2="V${shortver2%% *}"
pl_id="${NewVer##* (L}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id="${NewVer##* (T}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id=""
else
pl_id="${pl_id%%)*}"
pl_lang="${pl_id##*_}"
lang_level="${pl_id%%_*}"
pl_id=" (T${pl_id})"
this_major=`awk '/EQL_MAJREV/{print $3}' /build.c`
this_minor=`awk '/EQL_MINREV/{print $3}' /build.c`
lang_revmin=`awk '/LANG_REV_MIN/{print $3}' /build.c`
lang_revrec=`awk '/LANG_REV_REC/{print $3}' /build.c`
lang_major=`awk '/EQL_MAJREV/{print $3}' build.c`
lang_minor=`awk '/EQL_MINREV/{print $3}' build.c`
if [ "${this_major}" != "${lang_major}" -o "${this_minor}" != "${lang_minor}" ]; then
uabort "58:This is a language kit only for version \${1}" "V${lang_major}.${lang_minor}"
fi
if [ ${lang_level} -lt ${lang_revmin} ]; then
uabort "59:This language kit version \${1} is below the minimum version \${2}" "V${lang_major}.${lang_minor}${pl_id}" "V${this_major}.${this_minor} (T${lang_revmin})"
fi
if [ ${lang_level} -lt ${lang_revrec} ]; then
${LOGEVENT} 30.3.3 "V${lang_major}.${lang_minor}${pl_id}" "V${this_major}.${this_minor} (T${lang_revrec})"
log_msg "This language kit version V${lang_major}.${lang_minor}${pl_id} is below the recommended version V${this_major}.${this_minor} (T${lang_revrec})."
fi
fi
else
pl_id="${pl_id%%)*}"
pl_id=" (L${pl_id})"
if [ "${shortver1}" != "${shortver2}" ]; then
uabort "8:This is a patch kit only for version \${1}" "${shortver2}"
fi
fi
basenum="5000"
case "${cmtype}" in
70-0001) cmname="Type 1"
defplat="PSA"
;;
70-0011) cmname="Type 2"
defplat="PSA"
;;
70-0101) cmname="Type 3" ;;
70-0111) cmname="Type ${cmtype2}" ;;
70-0115) cmname="Type 5" ;;
70-0202) cmname="Type 7"; basenum="6000" ;;
70-0120) cmname="Type 8"; basenum="4000" ;;
70-0210) cmname="Type 9"; basenum="4000" ;;
70-0300) cmname="Type 10"; basenum="6010" ;;
70-0400) cmname="Type 11"; basenum="6100" ;;
70-0476) cmname="Type 12"; basenum="4100" ;;
70-0450) cmname="Type 13"; basenum="4110" ;;
70-0477) cmname="Type 14"; basenum="6110" ;;
70-0425) cmname="Type 15"; basenum="6030" ;;
70-0478) cmname="Type 17"; basenum="4110" ;;
70-0480) cmname="Type 18"; basenum="6610" ;;
70-0485) cmname="Type 19"; basenum="4210" ;;
esac
if [ -z "${platform}" ]; then
platform="${defplat}"
fi
case "${platform}" in
PSA) platformname="PS50 to PS400 series" ;;
RS1603) platformname="PS${basenum} series" ;;
SUMO) platformname="PS$((${basenum}+500)) series" ;;
BLACK_PEARL) platformname="PS${basenum} series" ;;
INTERCEPTOR) platformname="PS${basenum} series" ;;
FLYING_DUTCHMAN) platformname="PS${basenum} series" ;;
COLOSSUS_STORAGE_BLADE) platformname="PS-M${basenum} series" ;;
SHOGUN) platformname="PS$((${basenum}+500)) series" ;;
QUEEN_ANNES_REVENGE) platformname="PS${basenum} series" ;;
esac
KIT64=$(echo ${GZFile} | awk '/kit_64/')
log_msg "Controller type: ${cmtype} (${cmname})"
log_msg "Platform type:   ${platform} (${platformname})"
log_msg "Supported CMs in kit: ${supp_cms}"
log_msg "Supported platforms in kit: ${supp_platforms}"
cmok=
plok=
for c in ${supp_cms}; do
if [ "${c}" = "${cmtype}" ]; then
cmok="yes"
break
fi
done
for p in ${supp_platforms}; do
if [ "${p}" = "${platform}" ]; then
plok="yes"
break
fi
done
if [ -z "${cmok}" ]; then
unsupp_cms=$(awk '/Unsupported_Controllers/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
cmok=
for c in ${unsupp_cms}; do
if [ "${c}" = "${cmtype}" ]; then
cmok="yes"
break
fi
done
if [ -z "${cmok}" ]; then
uabort "9:The \${1} controller is not supported by this firmware.  See the latest Release Notes for more information" "${cmname}"
else
if [ -n "${KIT64}" ]; then
KITNEEDED="32"
else
KITNEEDED="64"
fi
uabort "71:For the \${1} controller, the \${2}-bit firmware kit is needed" "${cmname}" "${KITNEEDED}"
fi
fi
if [ -z "${plok}" ]; then
unsupp_platforms=$(awk '/Unsupported_Platforms/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
plok=
for p in ${unsupp_platforms}; do
if [ "${p}" = "${platform}" ]; then
plok="yes"
break
fi
done
if [ -z "${plok}" ]; then
uabort "10:This PS series array is not supported by this firmware.  See the latest Release Notes for more information"
else
if [ -n "${KIT64}" ]; then
KITNEEDED="32"
else
KITNEEDED="64"
fi
uabort "72:For this PS series array, the \${1}-bit firmware kit is needed" "${KITNEEDED}"
fi
fi
cd ${here}
}
check_firmware_version()
{
if [ -n "$TwoCMs" ]; then
primary_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /build.c)
get_secondary_ver
if [ -z "${secondary_ver}" ]; then
log_msg "SECONDARY_VER IS NIL: ${secondary_ver}"
sleep 3
get_secondary_ver
log_msg "SECONDARY_VER on retry: ${secondary_ver}"
fi
if [ "${primary_ver}" != "${secondary_ver}" ]; then
info_msg "Active:    ${primary_ver}"
info_msg "Secondary: ${secondary_ver}"
if [ -n "${GUI_MODE}" ]; then
uabort "11:Version mismatch between primary and secondary controllers"
fi
info_msg "At this point your only option is to update the secondary controller"
info_msg "to the same firmware version as the active controller."
while :
do
read -p "Do you want to proceed (y/n) [y]: " Answer
case "${Answer}" in
y | yes | "") echo
break
;;
n | no) echo "Update aborted."
if [ -n "${LOCKFILE}" ]; then
rm -f ${LOCKFILE}
fi
exit 1
;;
*) ;;
esac
done
force_secondary_version_match
get_secondary_ver
if [ "${primary_ver}" != "${secondary_ver}" ]; then
info_msg "Active:    ${primary_ver}"
info_msg "Secondary: ${secondary_ver}"
uabort "12:Firmware update to secondary control module unsuccessful.  Contact the support provider for your array"
else
info_msg "Forced update of secondary controller was successful"
fi
fi
fi
}
id_and_unpack_kit()
{
case "${B2BStat}" in
*Active?CM*)
ActiveCM="Yes"
;;
*)
info_msg "This command can only be run on the active control module."
exit 1
;;
esac
cd ${WDir}
rm -f eqlstor.gz eqlqrq.gz *.tar *.bin build.c cksums cfcksum* cksum_*
progress_msg 1
if [ -z "${GUI_MODE}" ]; then
ArrayName=`hostname`
if [ "$ArrayName" = "" ]
then
echo
echo "This command will install the update kit file that was"
echo "copied to the array."
echo
else
ArrayGroup=`psg_test -l | awk '{print $11}' | sed 's/\[//g' | sed 's/\]//g'`
echo
echo "This command will install the update kit file that was"
echo "copied to the member $ArrayName in the group $ArrayGroup."
echo
fi
echo "If you choose to proceed, you will be shown the current firmware version"
echo "and the version to which you will update.  You will then be given the"
echo "choice to proceed again."
echo
while :
do
read -p "Do you want to proceed (y/n) [y]: " Answer
case "${Answer}" in
y | yes | "") echo
break
;;
n | no) echo "Update aborted."
if [ -n "${LOCKFILE}" ]; then
rm -f ${LOCKFILE}
fi
exit 1
;;
*) ;;
esac
done
fi
cd ${WDir}
info_msg "Verifying kit integrity."
NameCk=${GZFile#kit_}
NameCk=${NameCk%%.tgz}
NameCk=${NameCk##*_}
retryable 'FileCk=$(cksum ${GZFile} 2>/dev/null)' "computing cksum of kit"
FileCk=${FileCk%% *}
if [ "${NameCk}" != "${FileCk}" ]; then
uabort "15:Checksum failure in kit file: aborting update, Nameck: \${1}, FileCk: \${2}" "${NameCk}" "${FileCk}"
fi
progress_msg 2
cleanup nolock
if [ -n "$(ls *.dgo 2>/dev/null)" ]; then
rm -f *.dgo
fi
required_files="build.c|cksums|update.sh"
gzip -dc "${WDir}/${GZFile}" 2>/dev/null | tar tvf - > /tmp/tar.lst
NEEDED_MB=`cat /tmp/tar.lst | awk "/${required_files}/" | awk '{s=int(($3+1023)/1024); x=x+s} END{print int((x+1023)/1024)}'`
AVAIL_MB=$(df -m /mgtdb| awk '/eqliomlv/{print $4}')
log_msg "Available space on /mgtdb ${AVAIL_MB}MB"
log_msg "Minimum set of files has a size of ${NEEDED_MB}"
if [ ${NEEDED_MB} -gt ${AVAIL_MB} ]; then
info_msg "Insufficient free space to extract the minimum set of files."
info_msg
if [ -n "$(ls *.core 2>/dev/null)" ]; then
info_msg "There are saved corefiles; consider using the \"diag\" command"
info_msg "to retrieve them for PS series support."
else
info_msg "Check the FTP upload directory for extraneous files."
fi
uabort "16:Insufficient free space to extract the minimum set of files; required: \${1}MB, available: \${2}MB" "${NEEDED_MB}" "${AVAIL_MB}"
fi
retryable 'extract_file "${WDir}/${GZFile}" cksums ${WDir}' "extract cksums"
cksumCk=$(awk '/ cksum_/ {print}' /tmp/tar.lst)
rm -f /tmp/tar.lst
if [ -n "${cksumCk}" ]; then
cksumCk=${cksumCk#*cksum_}
retryable 'FileCk=$(cksum cksums 2>/dev/null)' "computing cksum of cksums file"
FileCk=${FileCk%% *}
if [ "${cksumCk}" != "${FileCk}" ]; then
uabort "75:Checksum failure for cksums file: aborting update, cksumCk: \${1}, FileCk: \${2}" "${cksumCk}" "${FileCk}"
fi
fi
if [ -n "$(awk '/ update.sh/ {print}' ${WDir}/cksums 2>/dev/null)" ]; then
retryable 'extract_file_and_check "${WDir}/${GZFile}" update.sh ${WDir}' "extract and check update.sh"
else
retryable 'extract_file "${WDir}/${GZFile}" update.sh ${WDir}' "extract update.sh"
fi
retryable 'extract_file_and_check "${WDir}/${GZFile}" build.c ${WDir}' "extract and check build.c"
NewVer=$(awk '/EqualLogic_Version/{sub(".*= \x22", "");sub(".;$", "" );print}' \
/mgtdb/update/build.c)
shortver2="${NewVer##* V}"
shortver2="V${shortver2%% *}"
}
checkCFspace()
{
if [ -n "${pl_id}" ]; then
NEEDED_MB=$(cat ${WDir}/cksums | awk 'BEGIN{x=1} {s=int(($2+1023)/1024); x=x+s} END{print int((x+1023)/1024)}')
else
required_files="eqlstor.gz|eqlqrq.gz|${BUILD_FILE}|cksums|update.sh"
if [ -n "${cemi_file}" ]; then
required_files="${required_files}|${cemi_file}"
fi
if [ -n "${cemi2_file}" ]; then
required_files="${required_files}|${cemi2_file}"
fi
if [ -n "${ucemi_file}" ]; then
required_files="${required_files}|${ucemi_file}"
fi
if [ -n "${sep_file}" ]; then
required_files="${required_files}|${sep_file}"
fi
if [ -n "${sem_file}" ]; then
required_files="${required_files}|${sem_file}"
fi
if [ -n "${ipsec_file}" ]; then
required_files="${required_files}|${ipsec_file}"
fi
NEEDED_MB=`cat ${WDir}/cksums | awk "/${required_files}/" | awk 'BEGIN{x=1} {s=int(($2+1023)/1024); x=x+s} END{print int((x+1023)/1024)}'`
fi
if [ -z "${pl_id}" ]; then
if [ -d ${CFRoot}/update ]; then
if [ "$(ls ${CFRoot}/update/)" != "" ]; then
retryable "rm -rf ${CFRoot}/update/*" "rm -rf ${CFRoot}/update/*"
fi
fi
if [ -d ${CFRoot}/dumps ]; then
if [ "$(ls ${CFRoot}/dumps/)" != "" ]; then
retryable "rm -rf ${CFRoot}/dumps/*" "rm -rf ${CFRoot}/dumps/*"
fi
fi
fi
PRI_AVAIL_KB=$(df -k /pss| awk '/eqlfw/{print $4}')
PRI_TOT_KB=$(df -k /pss| awk '/eqlfw/{print $2}')
if [ -z "${pl_id}" ]; then
retryable "BootPathLine=$(awk '/BOOTPATH/{sub("\r$","");print}' ${CFRoot}/config.txt 2>/dev/null)" "get BootPathLine"
BootPath=${BootPathLine#BOOTPATH=}
if [ "${BootPath}" != "backup" ]; then
if [ -d ${CFRoot}/backup ]; then
BACKUP_AMOUNT="$(ls -l ${CFRoot}/backup | awk '/total/{print $2}')"
if [ -n "${BACKUP_AMOUNT}" ]; then
PRI_AVAIL_KB=$((${PRI_AVAIL_KB}+${BACKUP_AMOUNT}))
fi
fi
else
if [ -d ${CFRoot}/primary ]; then
PRIMARY_AMOUNT="$(ls -l ${CFRoot}/primary | awk '/total/{print $2}')"
if [ -n "${PRIMARY_AMOUNT}" ]; then
PRI_AVAIL_KB=$((${PRI_AVAIL_KB}+${PRIMARY_AMOUNT}))
fi
fi
fi
fi
PRI_EXPAND_FLASH=""
SEC_EXPAND_FLASH=""
if [ -z "${KIT64}" -a ${PRI_TOT_KB} -lt 200000 ]; then
PRI_AVAIL_KB=$((${PRI_AVAIL_KB}+180000))
PRI_EXPAND_FLASH="yes"
log_msg "Primary controller flash expansion will be done"
else
if [ -z "${KIT64}" ]; then
log_msg "Primary controller flash is already expanded"
else
log_msg "No flash expansion, 64 bit kit"
fi
fi
PRI_AVAIL_MB=$(((${PRI_AVAIL_KB}+1023)/1024))
log_msg "Space needed on the compact flash ${NEEDED_MB}MB"
log_msg "Available space on primary compact flash ${PRI_AVAIL_MB}MB"
if [ ${NEEDED_MB} -gt ${PRI_AVAIL_MB} ]; then
log_msg "primary flash contents:"
/bin/ls -alR ${CFRoot} >>${LOGFILE} 2>&1
uabort "19:Insufficient free space on primary compact flash; required: \${1}MB, available: \${2}MB" "${NEEDED_MB}" "${PRI_AVAIL_MB}"
fi
if [ -n "$TwoCMs" ]; then
if [ -z "${pl_id}" ]; then
tunnel "test -d ${CFRoot}/update"
if [ $? -eq 0 ]; then
tunnel "ls ${CFRoot}/update/" > /tmp/result.txt
if [ -s /tmp/result.txt ]; then
tunnel "rm -rf ${CFRoot}/update/*"
fi
fi
tunnel "test -d ${CFRoot}/dumps"
if [ $? -eq 0 ]; then
tunnel "ls ${CFRoot}/dumps/" > /tmp/result.txt
if [ -s /tmp/result.txt ]; then
tunnel "rm -rf ${CFRoot}/dumps/*"
fi
fi
fi
SEC_AVAIL_KB=$(tunnel "df -k /pss" | awk '/eqlfw/{print $4}')
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
SEC_TOT_KB=$(tunnel "df -k /pss" | awk '/eqlfw/{print $2}')
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ -z "${pl_id}" ]; then
retryable "SecBootPathLine=$(tunnel cat ${CFRoot}/config.txt | awk '/BOOTPATH/{sub("\r$","");print}')" "get SecBootPathLine"
SecBootPath=${SecBootPathLine#BOOTPATH=}
if [ "${SecBootPath}" != "backup" ]; then
tunnel "test -d ${CFRoot}/backup"
if [ $? -eq 0 ]; then
BACKUP_AMOUNT="$(tunnel "ls -l ${CFRoot}/backup" | awk '/total/{print $2}')"
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ -n "${BACKUP_AMOUNT}" ]; then
SEC_AVAIL_KB=$((${SEC_AVAIL_KB}+${BACKUP_AMOUNT}))
fi
fi
else
tunnel "test -d ${CFRoot}/primary"
if [ $? -eq 0 ]; then
PRIMARY_AMOUNT="$(tunnel "ls -l ${CFRoot}/primary" | awk '/total/{print $2}')"
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ -n "${PRIMARY_AMOUNT}" ]; then
SEC_AVAIL_KB=$((${SEC_AVAIL_KB}+${PRIMARY_AMOUNT}))
fi
fi
fi
fi
if [ -z "${KIT64}" -a ${SEC_TOT_KB} -lt 200000 ]; then
SEC_AVAIL_KB=$((${SEC_AVAIL_KB}+180000))
SEC_EXPAND_FLASH="yes"
log_msg "Secondary controller flash expansion will be done"
else
if [ -z "${KIT64}" ]; then
log_msg "Secondary controller flash is already expanded"
fi
fi
SEC_AVAIL_MB=$(((${SEC_AVAIL_KB}+1023)/1024))
log_msg "Available space on secondary compact flash ${SEC_AVAIL_MB}MB"
if [ ${NEEDED_MB} -gt ${SEC_AVAIL_MB} ]; then
SEC_PSS_CORE=$(tunnel "ls ${CFRoot}/*.cor* 2>/dev/null")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ -n "${SEC_PSS_CORE}" ]; then
info_msg "There are saved corefiles on secondary compact flash; consider using "
info_msg "the \"diag\" command to retrieve them for PS series support."
else
info_msg "Check the secondary compact flash for extraneous files."
fi
log_msg "secondary flash contents:"
tunnel "/bin/ls -alR ${CFRoot}" >>${LOGFILE} 2>&1
uabort "20:Insufficient free space on secondary compact flash; required: \${1}MB, available: \${2}MB" "${NEEDED_MB}" "${SEC_AVAIL_MB}"
fi
fi
}
checkRaidspace()
{
thismember="`hostname`"
mib_get "eqlGroupName"
gname="${MIBVAL}"
myindices=`LD_LIBRARY_PATH=/usr/lib:/mgtdb/update/mibwalk/lib /mgtdb/update/mibwalk/bin/snmpwalk -c public -M /mgtdb/update/mibwalk/mibs -m all -t 33 localhost eqlMemberRowStatus 2>/dev/null | awk -F . '{print $3}' | awk '{print $1}'`
if [ -n "${pl_id}" ]; then
return
fi
majver1="${shortver1%%.*}"
majver2="${shortver2%%.*}"
majver1="${majver1##V}"
majver2="${majver2##V}"
if [ ${majver1} -ne 6 -o $majver1 -le $majver2 ]; then
return
fi
if [ -z "${gname}" -o -z "${thismember}" ]; then
log_msg "Array is not setup"
return
fi
for i in ${myindices}; do
mib_get "eqlMemberName.1.${i}"
mname="${MIBVAL}"
if [ "${mname}" = "${thismember}" ]; then
mib_get "eqlDriveGroupRAIDPolicy.1.${i}"
log_msg "Raid Policy is ${MIBVAL}"
if [ "${MIBVAL}" = "unconfigured" ]; then
return
fi
fi
done
for i in ${myindices}; do
mib_get "eqlMemberName.1.${i}"
mname="${MIBVAL}"
if [ "${mname}" = "${thismember}" ]; then
mib_get "eqlMemberTotalStorage.1.${i}"
TotalStorage="${MIBVAL}"
mib_get "eqlMemberUsedStorage.1.${i}"
UsedStorage="${MIBVAL}"
log_msg "TotalStorage = ${TotalStorage}  UsedStorage = ${UsedStorage}"
if [ $((${TotalStorage}-${UsedStorage})) -ge 975 ]; then
break
fi
uabort "77:Insufficient storage space for this upgrade; required: 975MB, available: \${1}MB" "$((${TotalStorage}-${UsedStorage}))"
fi
done
return
}
checkGroupMembers()
{
thismember="`hostname`"
if [ -z "${gname}" -o -z "${thismember}" ]; then
log_msg "Array is not setup"
return
fi
cmp_version "${shortver2}" "${shortver1}"
if [ $ret -eq -1 ]; then
return
fi
if [ -n "${pl_id}" ]; then
return
fi
here=$PWD;
cd ${WDir}
supp_cms=$(awk '/Supported_Controllers/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
unsupp_cms=$(awk '/Unsupported_Controllers/{sub(".* \x22", "");sub("\x22;", "" );print }' build.c)
cd ${here}
myindices=`LD_LIBRARY_PATH=/usr/lib:/mgtdb/update/mibwalk/lib /mgtdb/update/mibwalk/bin/snmpwalk -c public -M /mgtdb/update/mibwalk/mibs -m all -t 33 localhost eqlMemberRowStatus 2>/dev/null | awk -F . '{print $3}' | awk '{print $1}'`
for i in ${myindices}; do
mib_get "eqlMemberName.1.${i}"
mname="${MIBVAL}"
if [ "${mname}" = "${thismember}" ]; then
continue
fi
mib_get "eqlMemberModel.1.${i}"
thisModel="${MIBVAL}"
cmok=
for c in ${supp_cms}; do
if [ "${c}" = "${thisModel}" ]; then
cmok="yes"
break
fi
done
if [ -z "${cmok}" ]; then
for c in ${unsupp_cms}; do
if [ "${c}" = "${thisModel}" ]; then
cmok="yes"
break
fi
done
if [ -z "${cmok}" ]; then
uabort "86:Member \${1} Model \${2} is not supported by this firmware.  See the latest Release Notes for more information" "${mname}" "${thisModel}"
fi
fi
done
return
}
umountCF()
{
UMountStat=
log_msg "vmstat -s | awk /times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}"
vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}" \
>>${LOGFILE}
loop_cnt=0
while :
do
sync
sync
UMountStat=$(umount ${CFRoot} 2>&1)
if [ -z "$UMountStat" ]; then
break
fi
if [ "${UMountStat}" = "umount: /pss: not currently mounted" ]; then
break;
fi
log_msg "umountCF: umount ${CFRoot} returned: ${UMountStat}, try#: ${loop_cnt}"
/bin/ps auxw >>${LOGFILE} 2>&1
if [ ${loop_cnt} -eq 10 ]; then
uabort "21:Failed to dismount the flash card on \${1}" "${CM_TYPE}"
fi
sleep 5
loop_cnt=$(($loop_cnt+1))
done
}
testFlash()
{
if [ ! -f ${CFRoot}/config.txt ]; then
uabort "22:Config.txt is missing on the compact flash card"
fi
umountCF
FSCK=/sbin/fsck_$(cat /dev/fs)
if [ ! -x ${FSCK} ]; then
log_msg "We do not have fsck available, so we are not testing the flash"
ChangeCFCardStatus ASYNC
return
fi
retry_cnt=0
while [ ${retry_cnt} -lt 10 ];
do
${FSCK} -y /dev/eqlfw >/tmp/fsck.out 2>&1
ret=$?
progress_msg 2
log_msg "fsck of ${CM_TYPE}:"
cat /tmp/fsck.out >> ${LOGFILE}
log_msg "fsck ret: ${ret} "
rm -f /tmp/fsck.out
if [ ${ret} -eq ${KILLED_BY_SIGTERM} ]; then
retry_cnt=$((${retry_cnt}+1))
log_msg "testFlash: fsck was killed, retry #${retry_cnt}"
continue
fi
if [ ${ret} -eq 0 ]; then
log_msg "testFlash: fsck returned success"
ChangeCFCardStatus ASYNC
return
else
log_msg "testFlash: fsck returned status ${ret}"
q8=$((${ret} / 8))
q4=$((${q8} / 4))
r4=$((${q8} - (${q4} * 4)))
if [ ${r4} -ne 0 ]; then
uabort "23:testFlash: fsck FATAL ret code of \${1}" "${ret}"
else
retry_cnt=$((${retry_cnt}+1))
log_msg "testFlash: fsck non-fatal ret code of ${ret}"
fi
fi
done
}
ChangeCFCardStatus()
{
if [ "$1" != "SYNC" \
-a "$1" != "ASYNC" ]
then
uabort "ChangeCFCardStatus on ${CM_TYPE}: invalid argument.  Received: $1 expected: SYNC or ASYNC"
fi
log_msg "ChangeCFCardStatus: to $1"
umountCF
if [ "$1" = "SYNC" ]
then
MountMode="-o sync"
else
MountMode=""
fi
MountMode="${MountMode} -o '-s'"
retryable "mount ${MountMode} -t $(cat /dev/fs) /dev/eqlfw ${CFRoot}" "mount cf $1"
return 0
}
extendcf()
{
BSW=$(awk '/BOOTPATH=backup/{print "-b"}' /pss/config.txt)
umountCF
log_msg $1 ${BSW} /dev/reqlfw
$1 ${BSW} /dev/reqlfw >> ${LOGFILE} 2>&1
ret=$?
if [ ${ret} -eq 0 ]; then
ChangeCFCardStatus ASYNC
return
else
ChangeCFCardStatus SYNC
uabort "84:Failure to extend the CompactFlash file system; please contact your Support Provider"
fi
}
prep_for_update()
{
curvers=$1
newvers=$2
info_msg "Preparing to update ${CM_TYPE} controller."
echo 'ftp		stream	tcp	nowait	root	/usr/libexec/ftpd	ftpd -ll' > /tmp/inetd.conf
echo 'telnet		stream	tcp	nowait	root	/usr/libexec/telnetd	telnetd' >> /tmp/inetd.conf
echo '127.0.0.1:telnet stream	tcp	nowait	root	/usr/libexec/telnetd	telnetd' >> /tmp/inetd.conf
echo '127.0.0.1:shell	stream	tcp	nowait:1000000	root	/usr/libexec/rshd	rshd -L' >> /tmp/inetd.conf
echo 'ssh		stream	tcp	nowait	root	/usr/sbin/sshd		sshd -i' >> /tmp/inetd.conf
mv /tmp/inetd.conf /etc/inetd.conf 2>/dev/null || rm -f /tmp/inetd.conf
/etc/rc.d/inetd reload >/dev/null
how_booted
if [ "${NetBoot}" = "Yes" ]; then
info_msg "Saving current network boot configuration on ${CM_TYPE} as \"config.nb\""
cp -f ${CFRoot}/config.txt ${CFRoot}/config.nb
fi
rm -rf /pss/dumps/*
if [ ! -d ${CFRoot}/primary ]; then
retryable "mkdir -p ${CFRoot}/primary" "mkdir -p ${CFRoot}/primary"
fi
if [ ! -d ${CFRoot}/backup ]; then
retryable "mkdir -p ${CFRoot}/backup" "mkdir -p ${CFRoot}/backup"
fi
if [ ! -d ${CFRoot}/update ]; then
retryable "mkdir -p ${CFRoot}/update" "mkdir -p ${CFRoot}/update"
fi
if [ "$(ls ${CFRoot}/update/)" != "" ]; then
retryable "rm -rf ${CFRoot}/update/*" "rm -rf ${CFRoot}/update/*"
fi
rootlist="tools mobiliti eqlcfe.gz"
fwlist="eqlcfe.gz eqlx"
here=$PWD
cd ${CFRoot}
for f in *; do
if file_matches $f $rootlist; then
log_msg "Getting rid of ${PWD}/${f}"
rm -rf $f
fi
done
cd ${CFRoot}/primary
for f in *; do
if file_matches $f $fwlist; then
log_msg "Getting rid of ${PWD}/${f}"
rm -rf $f
fi
done
cd ${here}
if [ "${NetBoot}" = "Yes" -o "${RenameDir}" = "Yes" ]; then
retryable "rm -rf ${CFRoot}/backup/*" "rm -rf ${CFRoot}/backup/*"
else
retryable "rm -rf ${CFRoot}/primary/*" "rm -rf ${CFRoot}/primary/*"
fi
}
updnote()
{
action=$1
for i in ${myindices}; do
mib_get "eqlMemberName.1.${i}"
mname="${MIBVAL}"
if [ ${action} = "prt" ]; then
version_get ${i} 1
slot1ver="${VERSION}"
version_get ${i} 2
slot2ver="${VERSION}"
if [ -n "${slot1ver}" ]; then
mver="${slot1ver}"
if [ -n "${slot2ver}" -a "${slot1ver}" != "${slot2ver}" ]; then
mver="MIX: ${slot1ver}, ${slot1ver}"
fi
else
mver="${slot2ver}"
fi
fi
mib_get "eqlBackplaneSN2.1.${i}" >/dev/null 2>&1
if [ -z "${MIBVAL}" ]; then
mib_get "eqlBackplaneSN.1.${i}"
fi
if [ -z "${MIBVAL}" ]; then
mib_get "eqlMemberSerialNumber.1.${i}" >/dev/null 2>&1
fi
mser="${MIBVAL}"
if [ -z "${MIBVAL}" ]; then
mser="unknown"
fi
if [ ${action} = "len" ]; then
namelen=${#mname}
if [ ${namelen} -gt ${maxname} ]; then
maxname=${namelen}
fi
serlen=${#mser}
if [ ${serlen} -gt ${maxser} ]; then
maxser=${serlen}
fi
fi
if [ ${action} = "prt" ]; then
if [ "${mname}" = "${thismember}" ]; then
thissn="${mser}"
mib_get "eqlMemberDescription.1.${i}"
thisdesc="${MIBVAL}"
fi
msg=`echo "${maxname} ${mname} ${maxser} ${mser}" | awk '{printf "  %-*s %-*s", $1, $2, $3, $4}'`
msg="${msg} ${mver}"
notice_msg "${msg}"
fi
done
}
how_booted()
{
if [ ! -r ${CFRoot}/config.txt ]; then
RenameDir="No"
NetBoot="No"
return
fi
BPLine=$(awk '/BOOTPATH/{sub("\r$","");print}' ${CFRoot}/config.txt)
case "${BPLine}" in
"BOOTPATH=primary") RenameDir="Yes"
NetBoot="No"
if [ ! -r /pss/primary/build.c ]; then
retryable "cp /build.c /pss/primary/build.c" "copy build.c to primary"
fi
;;
"BOOTPATH=backup") RenameDir="No"
NetBoot="No"
if [ ! -r /pss/backup/build.c ]; then
retryable "cp /build.c /pss/backup/build.c" "copy build.c to backup"
fi
;;
"BOOTPATH=") RenameDir="No"
NetBoot="Yes"
;;
*) uabort "25:BOOTPATH entry in \${1}/config.txt is invalid: \${2}" "${CFRoot}" "${BPLine}"
esac
}
spartanMPVCompatCheck()
{
if [ "${overrideMPV}" = "yes" ]; then
return
fi
mib_get_appl_indices
applIndices=${MIBVAL}
nasApplIndex=""
for applIndex in ${applIndices}; do
mib_get "eqlApplianceType.${applIndex}"
if [ ${MIBVAL} = "nas" ]; then
nasApplIndex=${applIndex}
fi
done
if [ -z ${nasApplIndex} ]; then
return
fi
mib_get "eqlApplianceState.${nasApplIndex}"
if [ -z ${MIBVAL} ]; then
uabort "Could not obtain NAS appliance state.  Update aborted"
fi
applianceState=${MIBVAL}
if [ -z ${CurEQLSpartanMPV} ]; then
uabort "28:Could not obtain current Management Protocol Version information"
fi
if [ -z ${NewEQLSpartanMPV} ]; then
uabort "29:Could not obtain update Management Protocol Version information"
fi
if [ ${applianceState} != "configured" ]; then
uabort "30:Nas-Group configuration is in progress. Update aborted"
fi
if [ ${CurEQLSpartanMPV} -eq ${NewEQLSpartanMPV} ]; then
return;
fi
if [ ${CurEQLSpartanMPV} -lt 2 ]; then
mib_get "eqlApplianceClusterMPV"
else
mib_get "eqlApplianceClusterMPV.${nasApplIndex}"
fi
if [ -z ${MIBVAL} ]; then
uabort "31:Filesystem appliance Management Protocol Version unknown.  Update aborted"
fi
SpartanClusterMPV=${MIBVAL}
if [ ${NewEQLSpartanMPV} -lt ${SpartanClusterMPV} ]; then
uabort "32:MPV of this firmware (\${1}) is less-than MPV of the currently-installed Spartan cluster (\${2})" "${NewEQLSpartanMPV}" "${SpartanClusterMPV}"
return
fi
if [ ${NewEQLSpartanMPV} -eq ${SpartanClusterMPV} ]; then
info_msg
info_msg "PLEASE NOTE:"
info_msg
info_msg "You are upgrading to Management Protocol Version ${NewEQLSpartanMPV},"
info_msg "which is equal to the MPV of currently-installed filesystem"
info_msg "appliance (${SpartanClusterMPV})."
info_msg "You will NOT be able to downgrade once the upgrade is finished."
info_msg
return;
fi
if [ ${NewEQLSpartanMPV} -eq $(($SpartanClusterMPV+1)) ]; then
return;
fi
if [ ${NewEQLSpartanMPV} -ge $(($SpartanClusterMPV+2)) ]; then
uabort "33:MPV of this firmware (\${1}) is two or more greater than MPV of currently-installed filesystem appliance (\${2})" "${NewEQLSpartanMPV}" "${SpartanClusterMPV}"
fi
info_msg ""
info_msg "Illegal program execution: ${applianceState} :: ${NewEQLSpartanMPV} :: ${CurEQLSpartanMPV} :: ${SpartanClusterMPV}"
info_msg ""
uabort "An internal error has occurred.  Update aborted"
}
spartanNASReserveCheck()
{
if [ "${overrideMPV}" = "yes" ]; then
return
fi
mib_get "eqlApplianceNumberOfNodes.${nasApplIndex}"
log_msg "Appliance Index = ${nasApplIndex}"
if [ -z ${MIBVAL} ]; then
log_msg "Filesystem appliance Number of Nodes unknown."
return
fi
SpartanNumberOfNodes=${MIBVAL}
mib_get "eqlApplianceStatsTotalCapacity.${nasApplIndex}"
if [ -z ${MIBVAL} ]; then
log_msg "Filesystem appliance Total Capacity unknown."
return
fi
SpartanTotalCapacity=${MIBVAL}
mib_get "eqlApplianceStatsTotalAllocated.${nasApplIndex}"
if [ -z ${MIBVAL} ]; then
log_msg "Filesystem appliance Total Allocated unknown."
return
fi
SpartanTotalAllocated=${MIBVAL}
TotalReserveFree=$((${SpartanTotalCapacity}-${SpartanTotalAllocated}))
ReserveNeeded=$((${SpartanNumberOfNodes}*76800))
log_msg "Spartan Number of Nodes = ${SpartanNumberOfNodes}"
log_msg "Spartan Total Reserve Free = ${TotalReserveFree}"
if [ ${ReserveNeeded} -le ${TotalReserveFree} ]; then
return
fi
TotalReserveGB=$(((${TotalReserveFree}+1023)/1024))
if [ ${SpartanNumberOfNodes} -eq 2 ]; then
NumOfNodes="two"
ReqGB="150"
else
NumOfNodes="four"
ReqGB="300"
fi
info_msg "Error: Upgrading the NAS controllers will fail because of"
info_msg "insufficient free NAS reserve space."
uabort "74:The NAS reserve has \${1} GB of free space. This cluster requires at least \${2} GB of free space. The upgrade cannot proceed" "${TotalReserveGB}" "${ReqGB}"
}
init_phase2()
{
LOCKFILE=""
if [ $# -gt 0 ]; then
if [ "$1" = "-lockfile" ]; then
LOCKFILE="$2"
shift 2
fi
fi
if [ -z "${LOCKFILE}" ]; then
interlock
fi
if [ "$1" = "-override-downgrade" ]; then
if [ "${USER}" = "root" ]; then
override_downgrade="yes"
log_msg "NOTE: -override-downgrade was specified"
shift 1
fi
fi
if [ "$1" = "-gui" ]; then
GUI_MODE="yes"
PROGRESS="$2"
shift 2
fi
if [ "$1" = "-override-MPV" ]; then
overrideMPV="yes"
log_msg "NOTE: -override-MPV was specified"
shift 1
fi
if [ "$1" = "-log" ]; then
LOG_ALL="yes"
log_msg "NOTE: -log was specified"
shift 1
fi
GZFile=
if [ "$1" = "-kit" ]; then
KIT_FILE="yes"
GZFile="$2"
shift 2
fi
if [ -r /mgtdb/update/build.c ]; then
NewVer=$(awk '/EqualLogic_Version/{sub(".*= \x22", "");sub(".;$", "" );print}' \
/mgtdb/update/build.c)
NewEQLSpartanMPV=$(awk '/ EQL_SPARTAN_MPV /{print $3}' /mgtdb/update/build.c)
else
if [ -r /mgtdb/update/ver ]; then
NewVer=$(cat /mgtdb/update/ver)
else
uabort "37:Cannot determine the firmware revision to which you are updating"
fi
fi
shortver2="${NewVer##* V}"
shortver2="V${shortver2%% *}"
pl_id="${NewVer##* (L}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id="${NewVer##* (T}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id=""
else
pl_id="${pl_id%%)*}"
pl_id=" (T${pl_id})"
group_lowest_cur_cl=$(/sbin/psgd_test -m | awk '/Group version/{sub(".*\\[",""); sub(",.*$",""); print }')
group_lowest_cur_cl=${group_lowest_cur_cl%.*}
if [ ${group_lowest_cur_cl} -lt 12 ]; then
log_msg "The lowest group compatibility level is ${group_lowest_cur_cl}"
uabort "70:The language kit requires that all group members run PS Series Firmware Version 6.0 or higher"
fi
fi
else
pl_id="${pl_id%%)*}"
pl_id=" (L${pl_id})"
fi
if [ -n "${GUI_MODE}" ]; then
if [ "${pl_id# (L}" != "${pl_id}" ]; then
prog_cnt=0
patch_opt="equall.jar eqlres.jar fxrplm.sh cemiup.sh b21523.sh"
for i in $patch_opt; do
if [ -n "$(cat cksums | awk "/ ${i}$/")" ]; then
prog_cnt=$((${prog_cnt}+12))
fi
done
if [ -n "$TwoCMs" ]; then
prog_cnt=$((${prog_cnt}*2))
prog_cnt=$((${prog_cnt}+74))
fi
prog_cnt=$((${prog_cnt}+77))
echo "Progressdata: ${prog_cnt}" >&2
else
if [ "${pl_id# (T}" != "${pl_id}" ]; then
if [ -n "$TwoCMs" ]; then
echo "Progressdata: 79" >&2
else
echo "Progressdata: 41" >&2
fi
else
if [ -n "$TwoCMs" ]; then
echo "Progressdata: 298" >&2
else
echo "Progressdata: 154" >&2
fi
fi
fi
fi
if [ -r build.c ]; then
BUILD_FILE=build.c
else
BUILD_FILE=ver
fi
cd ${WDir}
if [ -z "${GZFile}" ]; then
retryable 'GZList=$(ls -t kit_*.tgz 2>/dev/null)' "getting list of kits"
retryable 'NumGZs=$(echo $GZList 2>/dev/null | awk "{print NF}" 2>/dev/null)' "getting # of kits"
case "$NumGZs" in
0) uabort "34:No update kit files found in \${1} in Phase 2. Aborting update" "${PWD}"
;;
1) GZFile="${GZList}"
;;
*) GZFile=$(echo ${GZList} | awk '{print $1}')
;;
esac
fi
log_msg "Updating from kit file \"${GZFile}\""
verify_platform_compat
case "$(uname -m)" in
sbmips | xlrmips) if [ ! `gzip -dc ${WDir}/${GZFile} 2>/dev/null | tar tf - eqlqrq.gz` ]; then
uabort "17:Update kit does not have eqlqrq. Be sure the kit is for your system architecture"
fi
;;
*) uabort "18:Unknown architecture. Not yet supported"
;;
esac
log_msg "Contents of kit tarball:"
gzip -dc "${WDir}/${GZFile}" 2>/dev/null | tar tvf - >>${LOGFILE} 2>&1
log_msg "cksums file from kit:"
cat ${WDir}/cksums >>${LOGFILE} 2>&1
log_msg "build.c file from kit:"
cat ${WDir}/build.c >>${LOGFILE} 2>&1
progress_msg 2
cmp_version "${shortver2}" "${shortver1}"
if [ $ret -eq 1 ]; then
this_major=`awk '/EQL_MAJREV/{print $3}' /build.c`
this_minor=`awk '/EQL_MINREV/{print $3}' /build.c`
if [ ${this_major} -eq 8 -a ${this_minor} -eq 0 ]; then
mib_get "eqlStorageGroupStatusStorageContainerCount.1"
if [ -n "${MIBVAL}" ]; then
if [ ${MIBVAL} -gt 0 ]; then
vt=`awk '/EQL_VVOL_GROUP_SIZE_THRESHOLD/ { print $5 }' ${WDir}/build.c`
vt=${vt%;}
member_cnt=`psgd_test -x | awk 'BEGIN{cnt = 0} /uuid/ {cnt = cnt + 1} END {print cnt}'`
if [ ${member_cnt} -gt ${vt} ]; then
uabort "87:Cannot update this member. The maximum number of members for a VVol enabled group has been reached"
fi
fi
fi
fi
fi
if [ "$( /sbin/psgd_test -m | awk '/No entries in the PSG map/{print}')" != "No entries in the PSG map" ]; then
group_lowest_cur_cl=$(/sbin/psgd_test -m | awk '/Group version/{sub(".*\\[",""); sub(",.*$",""); print }')
group_highest_cur_cl=$(/sbin/psgd_test -m | awk '/Group version/{sub(".*, ",""); sub("\\]$",""); print }')
if [ -s ${group_highest_cur_cl} ]; then
group_lowest_cur_cl=1
group_highest_cur_cl=1
fi
if [ -r build.c ]; then
new_comp_level=$(awk '/EQL_COMP_LEVEL_CURRENT/{ sub(".*= ",""); sub( ",$", ""); print}' build.c)
if [ -z ${new_comp_level} ]; then
new_comp_level=1
else
jura_hack=$(awk '/EqualLogic_FullVersion.*@.#. 2.1.0.*/' build.c)
if [ -n "${jura_hack}" ]; then
new_comp_level=3
fi
fi
else
new_comp_level=1
fi
if [ ${group_highest_cur_cl##*.} = "1" ]; then
if [ ${group_highest_cur_cl} != ${group_lowest_cur_cl} ]; then
if [ "${group_lowest_cur_cl}" != "5.0" ]; then
uabort "39:Please call your PS Series support provider. The compatibility levels are not valid"
fi
fi
lowest_allowable_cl=${group_lowest_cur_cl%.[0-1]}
highest_allowable_cl=$((${group_highest_cur_cl%.1}+1))
else
lowest_allowable_cl=$((${group_highest_cur_cl%.[0-1]}-1))
highest_allowable_cl=$((${group_lowest_cur_cl%.[0-1]}+1))
fi
log_msg "Currently group compatibility level is:    ${group_lowest_cur_cl}..${group_highest_cur_cl}"
log_msg "Allowable group compatibility level is:    ${lowest_allowable_cl}..${highest_allowable_cl}"
log_msg "Upgrade will make new compatibility level: ${new_comp_level}"
notice_msg "Compat. level before update was:   ${group_lowest_cur_cl}..${group_highest_cur_cl}"
notice_msg "Allowable group compat. level was: ${lowest_allowable_cl}..${highest_allowable_cl}"
notice_msg "Post-update compatibility level:   ${new_comp_level}"
invalid_downgrade=""
if [ ${new_comp_level} -lt ${lowest_allowable_cl} -o ${new_comp_level} -gt ${highest_allowable_cl} ]; then
info_msg "Currently group compatibility level is:    ${group_lowest_cur_cl} through ${group_highest_cur_cl}"
info_msg "Allowable group compatibility level is:    ${lowest_allowable_cl} through ${highest_allowable_cl}"
info_msg "Upgrade would make new compatibility level: ${new_comp_level}"
invalid_downgrade="yes"
if [ -z "${override_downgrade}" ]; then
if [ ${new_comp_level} -lt ${lowest_allowable_cl} ]; then
uabort "40:You cannot downgrade to this firmware version because of compatibility issues. Update aborted"
else
info_msg "Cannot update the firmware.  The member does not meet the update path"
info_msg "requirements specified in the Release Notes.  In some cases, you need"
info_msg "to update to an interim version on all the group members before"
uabort "41:You cannot upgrade to this firmware version because of compatibility issues. Update aborted"
fi
fi
fi
majver1="${shortver1%%.*}"
majver2="${shortver2%%.*}"
majver1="${majver1##V}"
majver2="${majver2##V}"
minver1="${shortver1%.*}"
minver1="${minver1#*.}"
minver2="${shortver2%.*}"
minver2="${minver2#*.}"
maintver1="${shortver1##*.}"
maintver2="${shortver2##*.}"
if [ ${majver2} -eq ${majver1} -a ${majver2} -eq 9 ]; then
if [ ${minver2} -lt ${minver1} -a ${minver2} -eq 0 -a ${maintver2} -lt 7 ]; then
invalid_downgrade="yes"
if [ -z "${override_downgrade}" ]; then
uabort "80:You cannot downgrade from firmware version \${1} to \${2} because of compatibility issues. Update aborted" "${shortver1}" "${shortver2}"
fi
fi
fi
fi
cd /mgtdb/update
cemi_file=
if [ -n "${CEMI_TYPE}" ]; then
cemi_file=`awk "/ ${CEMI_TYPE}.*\.bin$/ {print}" cksums | awk '{print $3}'`
fi
cemi2_file=
if [ -n "${CEMI2_TYPE}" ]; then
cemi2_file=`awk "/ ${CEMI2_TYPE}.*\.bin$/ {print}" cksums | awk '{print $3}'`
fi
ucemi_file=
if [ -n "${UCEMI_TYPE}" ]; then
ucemi_file=`awk '/\.cem$/ {print $3}' cksums`
fi
sep_file=
if [ -n "${SEP_TYPE}" ]; then
sep_file=`awk "/ ${SEP_TYPE}.*\.bin$/ {print}" cksums | awk '{print $3}'`
fi
sem_file=
if [ -n "${SEM_TYPE}" ]; then
sem_file=`awk "/ ${SEM_TYPE}.*\.bin$/ {print}" cksums | awk '{print $3}'`
fi
ipsec_file=
if cm_supports_ipsec; then
ipsec_file=`awk "/ ipsec.rom$/ {print}" cksums | awk '{print $3}'`
fi
log_msg "cmtype ${cmtype} platform ${platform}"
log_msg "CEMI_TYPE ${CEMI_TYPE} CEMI2_TYPE ${CEMI2_TYPE} UCEMI_TYPE ${UCEMI_TYPE} SEP_TYPE ${SEP_TYPE} SEM_TYPE ${SEM_TYPE}"
log_msg "cemi_file ${cemi_file} cemi2_file ${cemi2_file} ucemi_file ${ucemi_file} sep_file ${sep_file} sem_file ${sem_file}"
log_msg "ipsec_file ${ipsec_file}"
checkCFspace
if [ -n "${PRI_EXPAND_FLASH}${SEC_EXPAND_FLASH}" ]; then
retryable 'extract_file_and_check "${WDir}/${GZFile}" extendcf ${WDir}' "extract and check extendcf"
fi
checkRaidspace
checkGroupMembers
oldIpcTunnel=$(ipctunnel -O 2>&1 | awk '/unknown option/{print}')
if [ -n "${TwoCMs}" ]; then
push2secondary ${UPDATE_SH} /tmp/update.sh
tunnel "chmod +x /tmp/update.sh"
UPDATE_SH2=/tmp/update.sh
tunnel "${UPDATE_SH2} remove_log_file"
if [ -n "${SEC_EXPAND_FLASH}" ]; then
push2secondary extendcf /tmp/extendcf
tunnel "chmod +x /tmp/extendcf"
tunnel "${UPDATE_SH2} remove_log_file"
fi
fi
progress_msg 1
if [ -n "${TwoCMs}" ]; then
info_msg "Active and secondary controllers detected."
log_msg "Stopping dbd on secondary controller"
tunnel "/sbin/eqlinit stop dbd" >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
tunnel "/sbin/eqlinit stop agent" >>${LOGFILE} 2>&1
tunnel "sync; ${UPDATE_SH2} testFlash"
ret=$?
if [ ${ret} -eq 88 ]; then
uabort "35:Cannot dismount the flash card on the secondary controller"
fi
else
info_msg
info_msg "Please note:"
info_msg
info_msg "Only one control module was detected.  If the array has two installed"
info_msg "control modules, do not perform the update until you correct the problem"
info_msg "that is preventing the array from detecting both control modules."
info_msg "Otherwise, only one control module will be updated."
info_msg
if [ -n "${GUI_MODE}" ]; then
uabort "36:Single controller updates are not allowed from the GUI. Correct the problem or run the update from the CLI"
fi
fi
mailserver=$(/sbin/eqllogger -M | awk '/SmtpServerAddress/ {print $2}' | awk '{sub("\\[",""); sub("\\]","");print}')
if [ "${mailserver}" = "0.0.0.0:25" -o "${mailserver}" = "255.255.255.255:25" ]; then
mailserver=
fi
eventmail=
localContact=
EMailAddr=
if [ -n "${mailserver}" ]; then
mib_get "eqlGroupEnableEmailNotifications"
if [ "${MIBVAL}" = "true" ]; then
mib_get "eqlGroupAlertEmailAddress"
for n in ${MIBVAL}; do
if [ -n "${eventmail}" ]; then
eventmail="${eventmail},$n"
else
eventmail="$n"
fi
done
fi
mib_get "eqlGroupEmailSupportContact"
if [ -n "${MIBVAL}" ]; then
localContact="${MIBVAL}"
fi
if [ -f /pss/identity/id.rc ]; then
EMailAddr=`awk '/^updatenotice.addr/ { print $3 }' /pss/identity/id.rc`
fi
if [ -z "${EMailAddr}" -a -n "${localContact}" ]; then
EMailAddr="update-notice@dell.com"
fi
if [ -n "${localContact}" ]; then
info_msg ""
info_msg "When the update procedure completes, the update procedure will"
info_msg "automatically send email to your PS Series support provider and to"
info_msg "the email address you have configured for your support contact:"
info_msg ""
info_msg "  ${localContact}"
EMailAddr="${EMailAddr},${localContact}"
if [ -n "$eventmail" ]; then
info_msg ""
info_msg "The update procedure will also send email to the email"
info_msg "addresses you have configured for event notification:"
info_msg ""
info_msg "  ${eventmail}"
EMailAddr="${EMailAddr},${eventmail}"
fi
info_msg ""
info_msg "Email notification requires connectivity from the array to the"
info_msg "Internet through the SMTP server at address ${mailserver}.  If"
info_msg "the SMTP server is not available or does not have access to the"
info_msg "Internet, you can forward the message from the support contact"
info_msg "email address to update-notice@dell.com."
else
if [ -n "${EMailAddr}" -o -n "${eventmail}" ]; then
info_msg ""
info_msg "To better support our customers, your PS Series support provider"
info_msg "requests that you notify us when you update storage array firmware."
info_msg ""
info_msg "When the update procedure completes, the update procedure"
if [ -n "${EMailAddr}" -a -n "${eventmail}" ]; then
info_msg "will automatically send email to the support provider:"
info_msg ""
info_msg "  ${EMailAddr}"
info_msg ""
info_msg "and to the email addresses you have configured for"
info_msg "event notification:"
info_msg ""
info_msg "  ${eventmail}"
EMailAddr="${EMailAddr},${eventmail}"
fi
if [ -n "${EMailAddr}" -a -z "${eventmail}" ]; then
info_msg "will automatically send email to the support provider:"
info_msg ""
info_msg "  ${EMailAddr}"
fi
if [ -z "${EMailAddr}" -a -n "${eventmail}" ]; then
info_msg "will automatically send email to the email addresses"
info_msg "you have configured for event notification:"
info_msg ""
info_msg "  ${eventmail}"
info_msg ""
info_msg "Email notification requires connectivity from the array to the"
info_msg "Internet through the SMTP server at address ${mailserver}.  If"
info_msg "the SMTP server is not available or does not have access to the"
info_msg "Internet, you can forward the message from one of the email"
info_msg "addresses configured for event notification to"
info_msg "update-notice@dell.com."
EMailAddr="${eventmail}"
fi
if [ -z "${GUI_MODE}" ]; then
while :
do
echo "Do you want to automatically send a notification email when"
echo -n "the update completes: (y/n) [y]: "
read Answer
case "${Answer}" in
y | yes | "") echo
break
;;
n | no) EMailAddr=
break
;;
*) ;;
esac
done
fi
fi
fi
if [ -n "$EMailAddr" ]; then
NOTICE_FILE=/mgtdb/update-notice.log
rm -f ${NOTICE_FILE}
notice_msg "Update completion notification"
notice_msg ""
notice_msg "Update started:    `date '+%Y.%m.%d %H:%M:%S %Z'`"
notice_msg "Group name:        ${gname}"
GRPADDR=$(psg_test -m | awk '/Master\ PSS\ ID/{print $8}' | awk '{sub(".*\\[",""); sub("\\]","");print}')
notice_msg "Group address:     http://${GRPADDR}/"
mib_get "eqlGroupDescription"
notice_msg "Group description: ${MIBVAL}"
thissn="unknown"
thisdesc=""
maxname=4
maxser=10
updnote len
maxname=$((${maxname}+2))
maxser=$((${maxser}-5))
notice_msg "Member list:"
msg=`echo "${maxname} Name Serial ${maxser} No. Version Before Update" | awk '{printf "  %-*s %s %-*s %s %s %s\n", $1, $2, $3, $4, $5, $6, $7, $8}'`
notice_msg "${msg}"
maxser=$((${maxser}+7))
updnote prt
notice_msg ""
notice_msg "Member being upgraded:             ${thismember}"
notice_msg "Member description:                ${thisdesc}"
mib_get "sysLocation"
notice_msg "Member location:                   ${MIBVAL}"
fi
fi
cd /mgtdb/update
how_booted
if [ "${NetBoot}" = "Yes" ]; then
info_msg
info_msg "Note: this array is currently configured for network booting."
if [ -n "${pl_id}" ]; then
uabort "57:This kit cannot be applied to a network-booted configuration"
fi
info_msg "The configuration will be changed to enable flash memory booting instead."
info_msg
fi
if [ "$NetBoot" = "No" ]
then
if [ "$RenameDir" = "Yes" ]
then
VerDir="${CFRoot}/primary"
else
VerDir="${CFRoot}/backup"
fi
if [ "${pl_id# (L}" != "${pl_id}" -a -r /var/patch/build.c ] ; then
VerDir="/var/patch"
fi
CurVer="unavailable."
VerFile=${VerDir}/build.c
if [ -f "$VerFile" -a -s "$VerFile" ]; then
CurVer=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' ${VerFile})
CurEQLSpartanMPV=$(awk '/ EQL_SPARTAN_MPV /{print $3}' ${VerFile})
else
VerFile=${VerDir}/ver
if [ -f "$VerFile" -a -s "$VerFile" ]; then
CurVer=$(cat ${VerFile})
fi
fi
else
if [ -e /build.c ]; then
CurVer=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' /build.c)
CurEQLSpartanMPV=$(awk '/ EQL_SPARTAN_MPV /{print $3}' /build.c)
else
CurVer=$(uname -v | awk '{print $11}')
fi
fi
info_msg
info_msg "   Currently running version is:   ${running_ver}"
info_msg "You will be updating to version:   ${NewVer}"
info_msg
notice_msg "Version before update:             ${running_ver}"
notice_msg "Version after update:              ${NewVer}"
if [ -n "${GUI_MODE}" ]; then
echo "Version before update:             ${running_ver}" >&2
echo "Version after update:              ${NewVer}" >&2
fi
notice_subj="PS Series Storage Array (Group ${gname} S/N ${thissn}) update from ${shortver1} to ${shortver2}${pl_id}"
majver1="${shortver1%%.*}"
majver2="${shortver2%%.*}"
majver1="${majver1##V}"
majver2="${majver2##V}"
if [ $majver1 -lt $majver2 ]; then
info_msg
info_msg "PLEASE NOTE:"
info_msg
info_msg "You are upgrading from firmware version ${shortver1} to version ${shortver2}."
info_msg "You will NOT be able to downgrade back to ${shortver1} once the"
info_msg "upgrade is finished."
info_msg
fi
if [ ${majver2} -eq ${majver1} -a ${majver2} -eq 9 ]; then
if [ ${minver2} -gt ${minver1} -a ${minver1} -eq 0 -a ${maintver1} -lt 7 ]; then
info_msg
info_msg "PLEASE NOTE:"
info_msg
info_msg "You are upgrading from firmware version ${shortver1} to version ${shortver2}."
info_msg "You will NOT be able to downgrade back to ${shortver1} once the"
info_msg "upgrade is finished."
info_msg
fi
fi
if [ -z "${pl_id}" ]; then
spartanMPVCompatCheck
if [ -n "${nasApplIndex}" ]; then
if [ ${CurEQLSpartanMPV} -lt 3 -a ${NewEQLSpartanMPV} -ge 3 ]; then
spartanNASReserveCheck
fi
fi
fi
if [ -z "${GUI_MODE}" -a -z "${pl_id}" ]; then
echo "The firmware update will not take effect until you restart the array."
echo
fi
if [ $majver2 -lt $majver1 ]; then
invalid_downgrade="yes"
if [ -z "${override_downgrade}" ]; then
uabort "42:You cannot downgrade from firmware version \${1} to version \${2}" "${shortver1}" "${shortver2}"
fi
fi
if [ -n "${invalid_downgrade}" ]; then
info_msg
info_msg "PLEASE NOTE:"
info_msg "You are doing a downgrade that is not allowed by the normal"
info_msg "version compatibility rules.  A \"reset\" will be run automatically"
info_msg "when the downgrade completes."
info_msg
fi
if [ -z "${GUI_MODE}" ]; then
while :
do
echo -n "Do you want to proceed (y/n) [n]: "
read Answer
case "${Answer}" in
y | yes) echo
break
;;
n | no | "") info_msg "You have elected to cancel the update"
cleanup_and_exit
;;
*) ;;
esac
done
fi
progress_msg 1
${LOGEVENT} 30.2.5 "${shortver2}${pl_id}"
/bin/ls -alR ${CFRoot} >> ${LOGFILE}
if [ "$(uname -r)" = "1.6.2" ]; then
disktool -c 0 > /dev/null 2>&1
(echo active:;df;vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}") >> ${LOGFILE}
KBLKS=$(vmstat -s | awk 'BEGIN {s=0;} /pages free/ && !/freed/ || /cached file/ {s+=$1;} END {print (s/2-256)*4}')
mkdir /var/tmp/scrfs
/sbin/mount_mfs -s ${KBLKS}k -i 16k /dev/null /var/tmp/scrfs > /dev/null 2>&1
KBLKS=$(df -k /var/tmp/scrfs | awk '/scrfs/ {print $4-32}')
dd bs=1024 count=$KBLKS if=/dev/zero of=/var/tmp/scrfs/scratch >> ${LOGFILE} 2>&1
umount /var/tmp/scrfs
rm -d /var/tmp/scrfs
(echo active:;df;vmstat -s | awk "/times.daemon.wokeup/,/pages.deactivated/{print} /pages.free$/,/pages.inactive/{print}") >> ${LOGFILE}
fi
testFlash
stty -isig
trap '' INT HUP QUIT TERM PIPE
if [ -z "${pl_id}" ]; then
prep_for_update ${shortver1} ${shortver2}
fi
if [ -n "${SEC_EXPAND_FLASH}" ]; then
info_msg "Expanding CF card file system on secondary controller."
if tunnel "${UPDATE_SH2} extendcf /tmp/extendcf"; then
true
else
uabort "85:CompactFlash expansion on secondary controller failed; please contact your Support Provider"
fi
fi
if [ -n "${PRI_EXPAND_FLASH}" ]; then
info_msg "Expanding CF card file system on active controller."
extendcf ./extendcf
fi
info_msg "Updating flash memory on active controller."
info_msg "This will take several minutes."
CFDestDir=update
if [ -z "${pl_id}" ]; then
cmp_version "${shortver2}" "${shortver1}"
if [ $ret -eq -1 ]; then
touch ${NDRUseRunningUpdate}
touch ${NDRDowngradeInProgress}
info_msg "Create Downdgrade In Progress file"
fi
fi
if [ "${pl_id# (L}" != "${pl_id}" ]; then
retryable "CFDestDir=$(cat ${CFRoot}/config.txt | awk '/BOOTPATH/{sub("\r$","");print}')" "get CFDestDir"
CFDestDir=${CFDestDir#BOOTPATH=}
if [ -z ${CFDestDir} ]; then
info_msg "Active controller booting from network."
info_msg "Unable to update active controller."
return
fi
if [ ! -e ${CFRoot}/${CFDestDir}/patch ]
then
retryable "mkdir ${CFRoot}/${CFDestDir}/patch" "mkdir patch directory"
fi
if [ ! -e /var/patch ]
then
retryable "mkdir /var/patch" "mkdir /var/patch directory"
fi
extract2local patch.rc ${CFRoot}/patch.rc
progress_msg 12
extract2local patch.sh ${CFRoot}/${CFDestDir}/patch/patch.sh
progress_msg 12
extract2local patchrom ${CFRoot}/${CFDestDir}/patch/patchrom
progress_msg 12
extract2local patchrom.asc ${CFRoot}/${CFDestDir}/patch/patchrom.asc
progress_msg 12
extract2local patch.inv ${CFRoot}/${CFDestDir}/patch/patch.inv
progress_msg 12
if [ -n "$(cat cksums | awk '/ equall.jar$/')" ]; then
extract2local equall.jar ${CFRoot}/${CFDestDir}/patch/equall.jar
progress_msg 12
fi
if [ -n "$(cat cksums | awk '/ eqlres.jar$/')" ]; then
extract2local eqlres.jar ${CFRoot}/${CFDestDir}/patch/eqlres.jar
progress_msg 12
fi
if [ -n "$(cat cksums | awk '/ fxrplm.sh$/')" ]; then
extract2local fxrplm.sh ${CFRoot}/${CFDestDir}/patch/fxrplm.sh
progress_msg 12
fi
if [ -n "$(cat cksums | awk '/ cemiup.sh$/')" ]; then
extract2local cemiup.sh ${CFRoot}/${CFDestDir}/patch/cemiup.sh
progress_msg 12
fi
if [ -n "$(cat cksums | awk '/ b21523.sh$/')" ]; then
extract2local b21523.sh ${CFRoot}/${CFDestDir}/patch/b21523.sh
progress_msg 12
fi
else
if [ "${pl_id# (T}" != "${pl_id}" ]; then
retryable "CFDestDir=$(cat ${CFRoot}/config.txt | awk '/BOOTPATH/{sub("\r$","");print}')" "get CFDestDir"
CFDestDir=${CFDestDir#BOOTPATH=}
if [ -z ${CFDestDir} ]; then
info_msg "Active controller booting from network."
info_msg "Unable to update active controller."
return
fi
if [ ! -e ${CFRoot}/${CFDestDir}/lang ]
then
retryable "mkdir ${CFRoot}/${CFDestDir}/lang" "mkdir lang directory"
fi
extract2local i18n-${pl_lang}.rom ${CFRoot}/${CFDestDir}/lang/i18n-${pl_lang}.rom
progress_msg 12
push2local build.c ${CFRoot}/${CFDestDir}/lang/build.c
mv ${CFRoot}/${CFDestDir}/lang/build.c ${CFRoot}/${CFDestDir}/lang/build-${pl_lang}.c
progress_msg 12
else
extract2local eqlstor.gz ${CFRoot}/${CFDestDir}/eqlstor.gz
progress_msg 5
extract2local eqlqrq.gz ${CFRoot}/${CFDestDir}/eqlqrq.gz
progress_msg 6
push2local ${BUILD_FILE} ${CFRoot}/${CFDestDir}/${BUILD_FILE}
progress_msg 1
push2local cksums ${CFRoot}/${CFDestDir}/cksums
progress_msg 1
if [ -n "${cemi_file}" ]; then
extract2local ${cemi_file} ${CFRoot}/${CFDestDir}/${cemi_file}
progress_msg 3
fi
if [ -n "${cemi2_file}" ]; then
extract2local ${cemi2_file} ${CFRoot}/${CFDestDir}/${cemi2_file}
progress_msg 3
fi
if [ -n "${ucemi_file}" ]; then
extract2local ${ucemi_file} ${CFRoot}/${CFDestDir}/${ucemi_file}
progress_msg 3
fi
if [ -n "${sep_file}" ]; then
extract2local ${sep_file} ${CFRoot}/${CFDestDir}/${sep_file}
progress_msg 3
fi
if [ -n "${sem_file}" ]; then
extract2local ${sem_file} ${CFRoot}/${CFDestDir}/${sem_file}
progress_msg 3
fi
if [ -n "${ipsec_file}" ]; then
extract2local ${ipsec_file} ${CFRoot}/${CFDestDir}/${ipsec_file}
progress_msg 4
fi
push2local update.sh ${CFRoot}/${CFDestDir}/update.sh
chmod +x ${CFRoot}/${CFDestDir}/update.sh
copy_lang_folder ${CFDestDir}
if [ -s /mgtdb/update/acctnam.lst ]; then
cp /mgtdb/update/acctnam.lst ${CFRoot}/${CFDestDir}/acctnam.lst
fi
if [ -s /mgtdb/update/sernum.bin ]; then
cp /mgtdb/update/sernum.bin ${CFRoot}/${CFDestDir}/sernum.bin
rm -f /mgtdb/update/sernum.bin
fi
fi
fi
log_msg "Flash contents on primary controller:"
/bin/ls -alR ${CFRoot} >>${LOGFILE} 2>&1
if [ -z "${pl_id}" ]; then
log_msg "cksums:"
cat ${CFRoot}/${CFDestDir}/cksums >>${LOGFILE} 2>&1
log_msg "build.c:"
cat ${CFRoot}/${CFDestDir}/build.c >>${LOGFILE} 2>&1
progress_msg 1
fi
testFlash
ChangeCFCardStatus SYNC
/bin/ls -alR ${CFRoot} >> ${LOGFILE}
if [ "${pl_id# (L}" != "${pl_id}" ]; then
${CFRoot}/${CFDestDir}/patch/patch.sh >>${LOGFILE} 2>&1
fi
if [ "${pl_id# (T}" != "${pl_id}" ]; then
log_msg "Mounting language pack"
umount /var/web/languages/${pl_lang} >>/dev/null 2>&1
mkdir -p /var/web/languages/${pl_lang} >>/dev/null 2>&1
mount -v -t romfs -o ro ${CFRoot}/${CFDestDir}/lang/i18n-${pl_lang}.rom /var/web/languages/${pl_lang} >>${LOGFILE} 2>&1
fi
info_msg "Update of flash memory on active controller completed."
if [ -n "$TwoCMs" ]; then
update_secondary_cm
fi
if [ -n "${pl_id}" ]; then
if [ -n "${EMailAddr}" ]; then
EMailFile=/tmp/mailit.sh
echo '#!/bin/sh' > ${EMailFile}
echo '. /sbin/sendEmail.sh' >> ${EMailFile}
echo "EMailAddr=\"${EMailAddr}\"" >> ${EMailFile}
echo "echo 'Sending update completion notice ...'" >> ${EMailFile}
echo "waitForNetwork 600 && sleep 30 && SendEMail '${notice_subj}' \"${NOTICE_FILE}\"" >> ${EMailFile}
echo 'rm -f $0' >> ${EMailFile}
chmod 755 ${EMailFile}
sh ${EMailFile} &
fi
cleanup
stty isig
trap INT HUP QUIT TERM PIPE
if [ -n "${KIT_FILE}" ]; then
rm -f /mgtdb/update/${GZFile}
else
rm -f /mgtdb/update/kit*
fi
${LOGEVENT} 30.2.6 "${shortver2}${pl_id}"
info_msg
info_msg "Update completed successfully."
info_msg
progress_msg 6
exit 0
fi
if [ -n "${EMailAddr}" ]; then
info_msg "Setting up for Email notification of update once new firmware has been loaded"
EMailFile=${NDREMailFile}
echo '#!/bin/sh' > ${EMailFile}
echo '. /sbin/sendEmail.sh' >> ${EMailFile}
echo "EMailAddr=\"${EMailAddr}\"" >> ${EMailFile}
echo "echo 'Sending update completion notice to your PS Series support provider...'" >> ${EMailFile}
echo "waitForNetwork 600 && sleep 30 && SendEMail '${notice_subj}' \"${NOTICE_FILE}\"" >> ${EMailFile}
echo 'rm -f $0' >> ${EMailFile}
chmod 755 ${EMailFile}
fi
cleanup
if [ -z "${invalid_downgrade}" ]; then
stty isig
trap INT HUP QUIT TERM PIPE
fi
ipc_id=`psg_test -m | awk 'BEGIN {t=0} /^uuid/ { t=0;
b=index($0,", id["); if (b>0) r=substr($0,b+5); e=index(r,"]");
if (e>0) n=substr(r,0,e-1); do g=getline; while ((index($0,"flags") == 0) && (g == 1));
s=index($0,"IPC_PSS_ONLINE");
if (s>0) t=index($0,"IPC_PSS_LOCAL");
if (t>0) printf "%d", n }'`
iom_ipc pending_update_version "${ipc_id}" >/dev/null 2>&1
rm -f /mgtdb/update/kit*
${LOGEVENT} 30.2.6 "${shortver2}${pl_id}"
info_msg
info_msg "Update completed successfully."
info_msg
if [ -n "${invalid_downgrade}" ]; then
info_msg "Since you did a downgrade that is outside the range of"
info_msg "normal version compatibility, a \"reset\" will now be done."
info_msg
ndr_update
if [ -n "$TwoCMs" ]; then
log_msg "Stopping dbd on secondary controller"
tunnel "/sbin/eqlinit stop dbd" >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
tunnel "/sbin/eqlinit stop agent" >>${LOGFILE} 2>&1
tunnel "${UPDATE_SH2} ndr_update"
fi
reset --force
exit 42
fi
info_msg "The firmware update will take effect when the array is restarted. To"
info_msg "restart the array, enter the restart command at the CLI prompt."
progress_msg 1
exit 0
}
ndr_verify_update_cksum()
{
if verify_checksums "${CFRoot}/update" ; then
echo "CHECKSUM RESULT: SUCCESS"
exit 1
else
echo "CHECKSUM RESULT: FAILED"
exit 0
fi
}
ndr_check_kit()
{
if ! verify_checksums "${CFRoot}/update" ; then
info_msg "Checksum verification failed on active CM"
return 0
fi
if [ -f ${CFRoot}/update/build.c ]; then
NewVer=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' ${CFRoot}/update/build.c)
activeVer="${NewVer##* V}"
activeVer="V${activeVer%% *}"
else
log_msg "No ${CFRoot}/update/build.c file on the active"
return 0
fi
if [ -n "$TwoCMs" ]; then
push2secondary /tmp/update.sh /tmp/update.sh
tunnel "chmod +x /tmp/update.sh"
tunnel "/tmp/update.sh ndr_verify_update_cksum" > /tmp/result.txt
if [ ! -s /tmp/result.txt ]; then
return 0
fi
stat=`awk '/CHECKSUM RESULT:/ {print $3; exit}' </tmp/result.txt`
if [ $stat != "SUCCESS" ]; then
cat /tmp/result.txt >>${LOGFILE} 2>&1
return 0
fi
tunnel "test -f ${CFRoot}/update/build.c"
if [ $? -eq 0 ]; then
NewVer=$(tunnel "awk '/EqualLogic_Version/{sub(\".* \x22\", \"\");sub(\"\x22;\", \"\" );print }' ${CFRoot}/update/build.c")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
passiveVer="${NewVer##* V}"
passiveVer="V${passiveVer%% *}"
else
log_msg "No ${CFRoot}/update/build.c file on the passive"
return 0
fi
if [ ${activeVer} != ${passiveVer} ]; then
log_msg "Version mismatch - Active: ${activeVer} - Passive: ${passiveVer}"
return 0
fi
fi
return 1
}
ndr_update()
{
NewVer=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' ${CFRoot}/update/build.c)
shortver2="${NewVer##* V}"
shortver2="V${shortver2%% *}"
pl_id="${NewVer##* (L}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id="${NewVer##* (T}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id=""
else
pl_id="${pl_id%%)*}"
pl_id=" (T${pl_id})"
fi
else
pl_id="${pl_id%%)*}"
pl_id=" (L${pl_id})"
fi
/sbin/ecli >/tmp/ecli.log 2>/dev/null <<- "EOF"
cmv
q
EOF
if [ -s /tmp/ecli.log ]; then
sn=`awk '/CM Serial Number:/ {print $4; exit}' </tmp/ecli.log`
else
sn="unknown"
fi
slot=`awk '/Version Information for / {print $4; exit}' </tmp/ecli.log`
slot=${slot%%:}
slot=`echo ${slot} | awk '{sub("CM", "");print }'`
${LOGEVENT} 30.2.7 ${shortver1} "${shortver2}${pl_id}" ${slot} ${sn}
if [ -e ${NDRHook} ]; then
log_msg "remove NDR hook file ${NDRHook}"
rm -f ${NDRHook}
fi
if [ -e ${NDRUseRunningUpdate} ]; then
log_msg "remove NDR running update file ${NDRUseRunningUpdate}"
rm -f ${NDRUseRunningUpdate}
fi
if [ -e ${NDRDowngradeInProgress} ]; then
info_msg "remove NDR downgrade in progress file ${NDRDowngradeInProgress}"
fi
if [ -e ${CFRoot}/ndrtmp ]; then
log_msg "remove leftover ndrtmp"
rm -rf ${CFRoot}/ndrtmp
fi
how_booted
if [ "${NetBoot}" = "Yes" ]; then
log_msg "currently booting from network"
if [ "$(ls ${CFRoot}/primary/)" != "" ]; then
log_msg "move backup directory to temp directory"
retryable "mv  ${CFRoot}/backup   ${CFRoot}/ndrtmp" "move backup to temp"
log_msg "move primary directory to backup directory"
retryable "mv  ${CFRoot}/primary   ${CFRoot}/backup" "move primary to backup"
else
log_msg "move primary directory to temp directory"
retryable "mv  ${CFRoot}/primary   ${CFRoot}/ndrtmp" "move primary to temp"
fi
else
if [ "${RenameDir}" = "Yes" ]; then
log_msg "currently booting from primary"
log_msg "move backup directory to temp directory"
retryable "mv  ${CFRoot}/backup   ${CFRoot}/ndrtmp" "move backup to temp"
log_msg "move primary directory to backup directory"
retryable "mv  ${CFRoot}/primary   ${CFRoot}/backup" "move primary to backup"
else
log_msg "currently booting from backup"
log_msg "move primary directory to temp directory"
retryable "mv  ${CFRoot}/primary   ${CFRoot}/ndrtmp" "move primary to temp"
fi
fi
log_msg "move update directory to primary directory"
retryable "mv     ${CFRoot}/update   ${CFRoot}/primary" "move update to primary"
if [ -d ${CFRoot}/ndrtmp ]; then
log_msg "remove contents of update temp directory"
retryable "rm -rf  ${CFRoot}/ndrtmp/*" "remove contents of update temp directory"
log_msg "move update temp directory to update directory"
retryable "mv     ${CFRoot}/ndrtmp   ${CFRoot}/update" "move update temp directory to update"
else
log_msg "create update directory"
retryable "mkdir -p ${CFRoot}/update" "mkdir -p ${CFRoot}/update"
fi
update_config
log_msg "Verify Flash"
sync; testFlash
log_msg "flash contents at update end:"
/bin/ls -alR ${CFRoot} >>${LOGFILE} 2>&1
${LOGEVENT} 30.2.8 ${shortver1} "${shortver2}${pl_id}" ${slot} ${sn}
}
get_passive_cm_slot()
{
echo "cmv" | ecli | awk '
/Version Information for/ {
if ($5 == "Passive") {
print substr($4, 3, 1)
exit
}
}
'
}
get_cemi_rev()
{
if [ "$1" = 0 ]
then
echo "ses revs" | ecli | awk '
/CEMI Slot A/ {
parts = split($6, data, /\./)
if (parts == 2)
printf("%02s%02s\n", data[1], data[2])
else
print "0000"
}
'
else
echo "ses revs" | ecli | awk '
/CEMI Slot B/ {
parts = split($6, data, /\./)
if (parts == 2)
printf("%02s%02s\n", data[1], data[2])
else
print "0000"
}
'
fi
}
ndr_cemi_check()
{
here=$PWD;
CEMI_SEP="CEMI"
if [ -n "${CEMI_TYPE}" ]
then
cd ${CFRoot}/update
cemi_file=
for f in ${CEMI_TYPE}*.bin; do if [ -f $f ]; then cemi_file=$f; fi; done
new_cemi_ver=${cemi_file%.bin}
new_cemi_ver=${new_cemi_ver#${CEMI_TYPE}}
log_msg "new_cemi_ver: $new_cemi_ver"
elif [ -n "${UCEMI_TYPE}" ]
then
cd ${CFRoot}/update
cemi_file=
for f in *.cem; do if [ -f $f ]; then cemi_file=$f; fi; done
new_cemi_ver=${cemi_file%.cem}
if enc_isa_sumo
then
new_cemi_ver=`echo $new_cemi_ver | sed 's/^....//'`
else
new_cemi_ver=`echo $new_cemi_ver | sed 's/....$//'`
fi
log_msg "UCEMI_TYPE: $UCEMI_TYPE, cemi_file: $cemi_file, new_cemi_ver: $new_cemi_ver"
elif [ -n "${SEP_TYPE}" ]
then
cd ${CFRoot}/update
cemi_file=
for f in ${SEP_TYPE}*.bin; do if [ -f $f ]; then cemi_file=$f; fi; done
new_cemi_ver=${cemi_file%.bin}
new_cemi_ver=${new_cemi_ver#${SEP_TYPE}}
log_msg "SEP_TYPE: $SEP_TYPE, cemi_file: $cemi_file, new_cemi_ver: $new_cemi_ver"
CEMI_SEP="SEP"
else
log_msg "No CEMI/SEP on this controller"
return 1
fi
cd ${here}
if [ -z "$new_cemi_ver" ]
then
log_msg "The new_cemi_ver could not be obtained."
return 1
fi
if [ -n "${SEP_TYPE}" ]
then
passive_cemi_ver=`get_active_slave_sep_version`
else
passive_slot=`get_passive_cm_slot`
passive_cemi_ver=`get_cemi_rev $passive_slot`
fi
if [ -z "$passive_cemi_ver" ]
then
log_msg "The passive_cemi_ver could not be obtained."
fi
log_msg "Checking $new_cemi_ver against $passive_cemi_ver on the passive."
if [ "${passive_cemi_ver}" != "${new_cemi_ver}" ]; then
case "${cmtype}" in
70-0001 | 70-0011) ETC=10 ;;
70-0115) ETC=20 ;;
70-0300) ETC=15 ;;
*) ETC=15 ;;
esac
info_msg
info_msg "PLEASE NOTE:"
info_msg "The restart process may take up to approximately ${ETC} minutes."
info_msg "During the restart process, do not restart or power down the array."
info_msg
tunnel "/tmp/update.sh do_cemi_update \"${CFRoot}/update/${cemi_file}\""
if [ -n "${SEP_TYPE}" ]
then
SEP_UPDATE="${new_cemi_ver}"
return
fi
else
ETC=10
info_msg
info_msg "PLEASE NOTE:"
info_msg "The restart process may take up to approximately ${ETC} minutes."
info_msg "During the restart process, do not restart or power down the array."
info_msg
fi
lretry_cnt=0
if [ -n "${SEP_TYPE}" ]
then
passive_cemi_ver=`get_active_slave_sep_version`
else
passive_cemi_ver=`get_cemi_rev $passive_slot`
fi
while [ "${passive_cemi_ver}" != "${new_cemi_ver}" ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 100 ]; then
info_msg "Waiting for secondary control module ${CEMI_SEP} update to version [${new_cemi_ver}] ..."
sleep 5
else
psg_test -A
uabort "44:Secondary control module \${1} update unsuccessful" "${CEMI_SEP}"
fi
if [ -n "${SEP_TYPE}" ]
then
passive_cemi_ver=`get_active_slave_sep_version`
else
passive_cemi_ver=`get_cemi_rev $passive_slot`
fi
done
}
firmware_msg()
{
uabort "76:Cannot update while member vacating"
uabort "89:Cannot start update because volume migration is in progress"
}
remove_sanmond()
{
if [ -f /mgtdb/rc.local ]; then awk !/rc.sanmond/ </mgtdb/rc.local >/mgtdb/rc.local.tmp; mv /mgtdb/rc.local.tmp /mgtdb/rc.local ; fi
if [ -f /mgtdb/rc.sanmond ]; then rm /mgtdb/rc.sanmond; fi
}
detect_cemi00()
{
enclosure=`echo 'cmv' | /sbin/ecli | /usr/bin/awk '/^Executing in enclosure type/ { print $5 ; exit }'`
if [ "${enclosure}" != "SUMO" ]
then
return
fi
part=`echo 'cmv' | /sbin/ecli | /usr/bin/awk '/^Plat type:/ { print $9 ; exit }'`
if [ "${part}" != "70-0300" ]
then
return
fi
num_disks=`echo 'sys' | /sbin/ecli | /usr/bin/awk '/^Number of Disks:/ { print $4 ; exit }'`
phys37=`/sbin/sastool -L | /usr/bin/awk 'BEGIN { count=0 } /Phys=37/ { count++ } END { print count }'`
if [ \( ${phys37} -ne 4 \) -o \( ${num_disks} -eq 16 \) ]
then
echo "Cemi condition detected"
fi
}
restart_update()
{
if [ "${CM_TYPE}" = "active" ]; then
if checkRAentries; then
true;
else
if [ -n "${GUI_MODE}" ]; then
uabort "62:The accelerated write cache on this member is very busy. The restart will not proceed. Use the CLI 'restart' command to continue and for more information"
fi
info_msg "The accelerated write cache on this member is very busy. The controller"
info_msg "restart will take longer than usual and may disrupt some applications."
info_msg "You can use SAN HeadQuarters to view the accelerated write cache usage."
info_msg "You may want to perform the restart at a time of less activity to avoid"
info_msg "application disruptions. If you are not using SAN HeadQuarters or need"
info_msg "more help, contact the support provider for your array."
while :
do
read -p "Do you want to continue with the controller restart now? (yes/no) [no]: " Answer
case "${Answer}" in
y | yes) echo
break
;;
n | no | "") echo "Restart aborted."
cleanup_and_exit
;;
*) ;;
esac
done
fi
fi
NewVer=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' ${CFRoot}/update/build.c)
shortver2="${NewVer##* V}"
shortver2="V${shortver2%% *}"
pl_id="${NewVer##* (L}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id="${NewVer##* (T}"
if [ "${pl_id}" = "${NewVer}" ]; then
pl_id=""
else
pl_id="${pl_id%%)*}"
pl_id=" (T${pl_id})"
fi
else
pl_id="${pl_id%%)*}"
pl_id=" (L${pl_id})"
fi
if [ "${CM_TYPE}" = "active" -a -n "$TwoCMs" ]; then
push2secondary /tmp/update.sh /tmp/update.sh
tunnel "chmod +x /tmp/update.sh"
tunnel "/tmp/update.sh detect_cemi00" > /tmp/result.txt
if [ -s /tmp/result.txt ]; then
uabort "83:Secondary Control Module redundancy has failed.  Please contact your Support Provider.  KCS# 623554"
fi
fi
rm -f /tmp/result.txt
info_msg "Verifying new firmware integrity."
if ndr_check_kit; then
uabort "64:Error verifying new firmware integrity"
fi
Cores=
for TFile in "/mgtdb/*.core" "/mgtdb/*.core.*" "/mgtdb/update/*.core" "/mgtdb/update/*.core.*"
do
Tmp=`echo $TFile`
if [ "$Tmp" != "$TFile" ]
then
Cores="$Cores $Tmp"
fi
done
for TFile in "/pss/dumps/*.core" "/pss/dumps/*.cor" "/pss/*.core" "/pss/*.cor" "/pss/*.COR"
do
Tmp=`echo $TFile`
if [ "$Tmp" != "$TFile" ]
then
Cores="$Cores $Tmp"
fi
done
if [ "$Cores" ]
then
for TFile in $Cores
do
log_msg "Removing ${TFile}"
rm -f $TFile
done
fi
if [ -n "${TwoCMs}" ]
then
Cores=
for TFile in "/pss/dumps/*.core" "/pss/dumps/*.cor" "/pss/*.core" "/pss/*.cor" "/pss/*.COR"
do
Tmp=$(tunnel "echo $TFile")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
if [ "$Tmp" != "$TFile" ]
then
Cores="$Cores $Tmp"
fi
done
if [ "$Cores" ]
then
for TFile in $Cores
do
log_msg "Removing ${TFile} on secondary"
tunnel "rm -f $TFile"
done
fi
fi
disktool -s 0 >> ${LOGFILE} 2>&1
raidtool > ${CFRoot}/update/raidtool.txt
raidtool -L all >> ${CFRoot}/update/raidtool.txt
if [ -n "${TwoCMs}" ]
then
push2secondary ${CFRoot}/update/raidtool.txt ${CFRoot}/update/raidtool.txt
fi
if [ -z "${TwoCMs}" ]; then
info_msg "Updating single CM system."
restart_update2
exit 0
fi
SEP_UPDATE=""
ndr_cemi_check
info_msg "Start update of flash memory on secondary controller."
psg_test -S
log_msg "Stopping dbd on secondary controller"
tunnel "/sbin/eqlinit stop dbd" >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
tunnel "/sbin/eqlinit stop agent" >>${LOGFILE} 2>&1
if [ -e ${NDRUseRunningUpdate} ]; then
retryable 'tunnel "cp /sbin/update.sh /tmp/update.sh"' "copy update.sh on 2ndary"
else
retryable 'tunnel "cp ${CFRoot}/update/update.sh /tmp/update.sh"' "copy update.sh on 2ndary"
fi
info_msg "Setting cache to write through"
if ! /sbin/cflush ; then
stty isig
trap INT HUP QUIT TERM PIPE
uabort "45:The cache flush operation failed"
fi
retryable 'tunnel "/tmp/update.sh ndr_update"' "NDR update on 2ndary"
log_msg "*** Secondary Update log ***"
tunnel "cat /tmp/secondary.update.log" >>${LOGFILE} 2>&1
tunnel "rm -f  /tmp/secondary.update.log"
tunnel "ls ${CFRoot}/update/" > /tmp/result.txt
if [ -s /tmp/result.txt ]; then
psg_test -A
uabort "46:Update of flash memory on secondary controller failed"
else
info_msg "Update of flash memory on secondary controller completed."
fi
info_msg "Restarting secondary controller."
tunnel NOCHECK "reboot" 2> /dev/null
lretry_cnt=0
sleep 10
if [ "$cmtype" = "70-0400" -o "$cmtype" = "70-0476" -o "$cmtype" = "70-0477" -o "$cmtype" = "70-0425" -o "$cmtype" = "70-0478" -o "$cmtype" = "70-0485" ]; then
lretry_max=51
else
lretry_max=35
fi
while [ "`psg_test -U | awk '{print $3}'`" != "1" ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt ${lretry_max} ]; then
info_msg "Waiting for secondary controller to restart..."
sleep 5
else
psg_test -A
uabort "47:Secondary control module reboot unsuccessful"
fi
done
if [ -e ${NDRDowngradeInProgress} ]; then
info_msg "Firmware downgrade is in progress..."
psg_test -D
fi
info_msg "Secondary controller successfully restarted.  Start secondary control module synchronization."
lretry_cnt=0
while [ "`psg_test -U | awk '{print $4}'`" != "1" ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 180 ]; then
if [ $((${lretry_cnt}/24*24)) -eq ${lretry_cnt} ]; then
info_msg "Waiting up to $((15-${lretry_cnt}/12)) minutes more for secondary control module synchronization"
else
info_msg "Waiting for secondary control module synchronization..."
fi
sleep 5
else
psg_test -A
uabort "48:Secondary control module synchronization unsuccessful"
fi
done
lretry_cnt=0
while [ "`psg_test -U | awk '{print $5}'`" != "1" ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 180 ]; then
if [ $((${lretry_cnt}/24*24)) -eq ${lretry_cnt} ]; then
info_msg "Waiting up to $((15-${lretry_cnt}/12)) minutes more for secondary control module database synchronization"
else
info_msg "Waiting for secondary control module database synchronization..."
fi
sleep 5
else
psg_test -A
uabort "49:Secondary control module database synchronization unsuccessful"
fi
done
lretry_cnt=0
tunnel NOCHECK "test -s /tmp/PrefetchDone"
ret=$?
while [ ${ret} -ne 0 ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 36 ]; then
info_msg "Waiting for secondary control module Prefetch synchronization..."
sleep 5
tunnel NOCHECK "test -s /tmp/PrefetchDone"
ret=$?
else
log_msg "Secondary control module Prefetch taking longer than 3 minutes."
break
fi
done
info_msg "iSCSI PR PPool synchronization .. "
iscsi_test -L 9999
lretry_cnt=0
while [ "`psg_test -U | awk '{print $7}'`" != "1" ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -le 10 ]; then
info_msg "Waiting for iSCSI PR PPool synchronization..."
sleep 1
else
info_msg "Synchronization of iSCSI PR PPool failed. Persistent reservations could be lost."
break
fi
done
if [ -n "${SEP_UPDATE}" ]; then
passive_cemi_ver=`get_active_slave_sep_version`
if [ "${passive_cemi_ver}" != "${SEP_UPDATE}" ]; then
log_msg "SEP did not update from version ${passive_cemi_ver} to ${SEP_UPDATE}"
fi
fi
info_msg "Secondary controller successfully updated.  Transition current active controller to secondary controller."
log_msg "create NDR hook file ${NDRHook}"
echo '#!/bin/sh' > ${NDRHookTmp}
if [ -e ${NDRUseRunningUpdate} ]; then
echo "cp /sbin/update.sh /tmp/update.sh" >> ${NDRHookTmp}
else
echo "cp /pss/update/update.sh /tmp/update.sh" >> ${NDRHookTmp}
fi
chmod +x ${NDRHookTmp}
log_msg "cat NDR hook file ${NDRHookTmp} start"
cat ${NDRHookTmp} >> ${LOGFILE}
log_msg "cat NDR hook file ${NDRHookTmp} complete"
push2local ${NDRHookTmp} ${NDRHook}
log_msg "cat NDR hook file ${NDRHook} start"
cat ${NDRHook} >> ${LOGFILE}
log_msg "cat NDR hook file ${NDRHook} complete"
remove_sanmond
info_msg "Restarting active controller to complete the update."
reboot -N
exit 0
}
restart_update2()
{
exec 1> /dev/console
exec 2> /dev/console
if [ -e ${NDRHook} ]; then
lretry_cnt=0
while [ -z "$TwoCMs" ]
do
B2BStat=$(cord -b)
case "${B2BStat}" in
*channel?Down*)
TwoCMs=""
log_msg "B2B Channel Down"
;;
*channel?Up*)
TwoCMs="Yes"
break
;;
*)
log_msg "Unknown number of control modules"
;;
esac
sleep 10
lretry_cnt=$((${lretry_cnt}+1))
if [ $lretry_cnt -ge 60 ]
then
uabort "50:No communication with the active control module; please call the support provider for your array"
fi
info_msg "restart_update2: waiting for the B2B Link. Trial: $lretry_cnt"
done
fi
if [ -n "$TwoCMs" ]; then
info_msg "Successful transition of old active controller to secondary controller.  Complete update of firmware on this controller."
lretry_cnt=0
sleep 10
while [ -z "`cord -b | awk /Comm\ State\:\ REQUEST_PROTOCOL_INFO./`" -a -z "`cord -b | awk /Comm\ State\:\ CORD_PPOOL_IN_SYNC/`" -a -z "`cord -b | awk /Comm\ State\:\ CONNECTED/`"]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 120 ]; then
info_msg "Waiting for active..."
sleep 5
else
uabort "51:No communication with the active control module; please call the support provider for your array"
fi
done
MOVE_UPDATE_LOG="Y"
lretry_cnt=0
sleep 5
b2bState=$(tunnel "cord -b")
status=$?
log_msg "cord -b status is $status"
while [ ${status} -ne 1 ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 20 ]; then
info_msg "Waiting for peer controller to become the active controller..."
sleep 30
else
uabort "52:Peer controller is not running as the active controller"
fi
b2bState=$(tunnel "cord -b")
status=$?
log_msg "cord -b output: ${b2bState}"
done
lretry_cnt=0
sleep 10
peer_running_ver=$(tunnel "awk '/EqualLogic_Version/{sub(\".* \x22\", \"\");sub(\"\x22;\", \"\" );print }' /build.c")
ret=$?
log_msg "peer_running_ver 1 =[${peer_running_ver}]"
log_msg "Status of get version command: ${ret}"
if [ ${ret} -eq 88 ]; then
exit 88
fi
while [ -z "${peer_running_ver}" -o ${ret} -ne 0 ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 120 ]; then
info_msg "Waiting for active..."
sleep 5
peer_running_ver=$(tunnel "awk '/EqualLogic_Version/{sub(\".* \x22\", \"\");sub(\"\x22;\", \"\" );print }' /build.c")
ret=$?
log_msg "peer_running_ver 1 =[${peer_running_ver}]"
log_msg "Status of get version command: ${ret}"
if [ ${ret} -eq 88 ]; then
exit 88
fi
else
uabort "65:Cannot obtain the firmware version of the active control module"
fi
done
peer_running_ver="${peer_running_ver##* V}"
log_msg "peer_running_ver 2 =[${peer_running_ver}]"
peer_running_ver="V${peer_running_ver%% *}"
log_msg "peer_running_ver 3 =[${peer_running_ver}]"
new_ver=$(awk '/EqualLogic_Version/{sub(".* \x22", "");sub("\x22;", "" );print }' ${CFRoot}/update/build.c)
new_ver="${new_ver##* V}"
new_ver="V${new_ver%% *}"
log_msg "Peer Controller running version ${peer_running_ver}  new firmware version ${new_ver}"
if [ "${peer_running_ver}" != "${new_ver}" ]; then
uabort "53:Active controller is not running the new version of the firmware"
fi
active_online=$(tunnel "raidtool|awk '/Driver Status:/ {if (\$3 == \"Ok\")print \"ON\"}'")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
lretry_cnt=0
while [ -z ${active_online} ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 120 ]; then
info_msg "Waiting for active to come online..."
sleep 5
active_online=$(tunnel "raidtool|awk '/Driver Status:/ {if (\$3 == \"Ok\")print \"ON\"}'")
ret=$?
if [ ${ret} -eq 88 ]; then
exit 88
fi
else
uabort "54:Active control module is not online; please call the support provider for your array"
fi
done
log_msg "Stopping dbd on secondary controller"
/sbin/eqlinit stop dbd >>${LOGFILE} 2>&1
log_msg "Stopping agent on secondary controller"
/sbin/eqlinit stop agent >>${LOGFILE} 2>&1
fi
info_msg "Start update of flash memory on controller."
info_msg "This will take several minutes."
if [ -n "${SEP_TYPE}" ]
then
here=$PWD;
cd ${CFRoot}/update
cemi_file=
for f in ${SEP_TYPE}*.bin; do if [ -f $f ]; then cemi_file=$f; fi; done
new_cemi_ver=${cemi_file%.bin}
new_cemi_ver=${new_cemi_ver#${SEP_TYPE}}
log_msg "SEP_TYPE: $SEP_TYPE, cemi_file: $cemi_file, new_cemi_ver: $new_cemi_ver"
cd ${here}
if [ -z "${TwoCMs}" ]; then
current_cemi_ver=`get_active_sep_version`
else
current_cemi_ver=`get_active_slave_sep_version`
fi
if [ -z "${new_cemi_ver}" -o -z "${current_cemi_ver}" ]
then
log_msg "The new_cemi_ver or current_cemi_ver could not be obtained."
else
log_msg "Checking $new_cemi_ver against $current_cemi_ver on the ${CM_TYPE}."
if [ "${current_cemi_ver}" != "${new_cemi_ver}" ]; then
log_msg "Upgrading SEP from ${current_cemi_ver} to ${new_cemi_ver}"
if [ -z "${TwoCMs}" ]; then
/sbin/sep_update -s ${CFRoot}/update/${cemi_file} >>${LOGFILE} 2>&1
ret=$?
else
/sbin/sep_update -R -s ${CFRoot}/update/${cemi_file} >>${LOGFILE} 2>&1
ret=$?
if [ ${ret} -eq 1 ]; then
/sbin/sep_update -s ${CFRoot}/update/${cemi_file} >>${LOGFILE} 2>&1
ret=$?
fi
fi
if [ ${ret} -ne 0 ]; then
log_msg "Return status of ${ret}"
fi
fi
fi
fi
ndr_update
exec 0< /dev/console
exec 1> /dev/console
exec 2> /dev/console
if [ -z "${TwoCMs}" ]; then
if [ -f ${NDREMailFile} ]; then
info_msg "Setting up for Email notification of update at next reboot."
mv ${NDREMailFile} /mgtdb/runonce.sh
fi
fi
info_msg "Update complete.  Rebooting controller to load new firmware."
if [ -n "$TwoCMs" ]; then
lretry_cnt=0
DEFLOGFILE="/mgtdb/update`date '+%Y%m%d%H%M%S'`.log"
cat ${LOGFILE} 2>/dev/null | tunnel INPUT "cat >${DEFLOGFILE}"
status=$?
while [ ${status} -ne 0 ]
do
lretry_cnt=$((${lretry_cnt}+1))
if [ ${lretry_cnt} -lt 20 ]; then
info_msg "Attempting to copy update log file to the active controller..."
sleep 30
fi
DEFLOGFILE="/mgtdb/update`date '+%Y%m%d%H%M%S'`.log"
cat ${LOGFILE} 2>/dev/null | tunnel INPUT "cat >${DEFLOGFILE}"
status=$?
log_msg "Copy status: ${status}"
done
fi
sync
sleep 10
if [ -n "$TwoCMs" ]; then
reboot
else
remove_sanmond
reboot -N
fi
exit 1
}
common_initialization
unset override_downgrade
if [ $# -gt 0 ]; then
if [ "$1" = "-override-downgrade" ]; then
if [ "${USER}" = "root" ]; then
override_downgrade="$1"
log_msg "NOTE: -override-downgrade was specified"
shift 1
fi
fi
fi
unset GUI_MODE
if [ $# -gt 0 ]; then
if [ "$1" = "-gui" ]; then
GUI_MODE="$1"
shift 1
fi
fi
if [ "${CM_TYPE}" = "active" ]; then
if [ -z "${GUI_MODE}" ]; then
if [ ! -t 0 -o ! -t 1 ]; then
GUI_MODE="-gui"
fi
fi
fi
if [ -n "${GUI_MODE}" ]; then
ulimit -d 524288
ulimit -s 32768
ulimit -m unlimited
ulimit -l unlimited
fi
unset overrideMPV
if [ $# -gt 0 ]; then
if [ "$1" = "-override-MPV" ]; then
overrideMPV="$1"
shift 1
fi
fi
unset LOG_ALL
if [ $# -gt 0 ]; then
if [ "$1" = "-log" ]; then
LOG_ALL="$1"
shift 1
fi
fi
unset P2
if [ "$1" = "-p2" ]; then
P2="$2"
shift 2
fi
PROGRESS=0
if [ $# -gt 0 ]; then
if [ "$1" != "init_phase2" ]; then
trap '' INT HUP QUIT TERM PIPE
fi
if [ -n "${ThisCM}" ]; then
log_msg "Starting on ${ThisCM}"
fi
log_msg "${UPDATE_SH} $*"
if [ -n "${GUI_MODE}" ]; then
log_msg "In GUI mode"
fi
eval $*
exit $?
fi
countof cemi_update; cemi_pri=$?
if [ -n "$TwoCMs" ]; then
countof cemi_update sec; cemi_sec=$?
else
cemi_sec=0
fi
if [ $cemi_pri -gt 0 -o $cemi_sec -gt 0 ]; then
uabort "55:A CEMI update is still in progress"
fi
LOCKFILE=""
interlock
if [ -n "${GUI_MODE}" ]; then
if [ -n "$TwoCMs" ]; then
echo "Progressdata: 298" >&2
else
echo "Progressdata: 154" >&2
fi
fi
purge_log_files
if [ -z "${ThisCM}" ]; then
ThisCM="CM"
fi
log_msg "${UPDATE_SH} starting on ${CM_TYPE} ${ThisCM}"
if [ -r /pss/config.txt ]; then
cp /pss/config.txt /tmp/config.old
fi
check_firmware_version
cd ${WDir}
GZList=$(ls -t kit_*.tgz 2>/dev/null)
retryable 'NumGZs=$(echo $GZList 2>/dev/null | awk "{print NF}" 2>/dev/null)' "getting # of kits"
case "$NumGZs" in
0) uabort "13:No update kit files found in \${1}. Aborting update" "${PWD}"
;;
1) GZFile="${GZList}"
;;
*) NumLang=0
NumFirm=0
for GZFile in ${GZList}; do
if [ -z "${GZFile#kit*-T*.tgz}" ]; then
NumLang=$((${NumLang}+1))
else
NumFirm=$((${NumFirm}+1))
fi
done
if [ ${NumLang} -gt 0 -a ${NumFirm} -gt 0 ]; then
uabort "69:Both firmware and language kits found in \${1}" "${PWD}"
fi
log_msg "Multiple kit files ${GZList}"
if [ ${NumFirm} -gt 0 ]; then
if [ -n "${GUI_MODE}" ]; then
uabort "14:Multiple firmware kit files found in \${1}" "${PWD}"
fi
GZFile=$(echo ${GZList} | awk '{print $1}')
info_msg "Warning: multiple firmware kit files found"
info_msg "Using newest one: \"${GZFile}\""
GZList=${GZFile}
fi
;;
esac
ln -s /sbin/psgd_test /tmp/psgd_test 2>/dev/null
for GZFile in ${GZList}; do
info_msg "Updating from kit file \"${GZFile}\""
id_and_unpack_kit
cmp_version "${shortver2}" "${shortver1}"
if [ $ret -eq -1 ]; then
log_msg "Using ${UPDATE_SH}"
else
UPDATE_SH=/mgtdb/update/update.sh
fi
log_msg "UPDATE_SH:  ${UPDATE_SH}"
log_msg "UPDATE_SH2: ${UPDATE_SH2}"
log_msg "Starting phase 2 using ${UPDATE_SH}"
if [ -n "${GUI_MODE}" ]; then
GUI_MODE="-gui ${PROGRESS}"
fi
${UPDATE_SH} init_phase2 -lockfile ${LOCKFILE} ${override_downgrade} ${GUI_MODE} ${overrideMPV} ${LOG_ALL} -kit ${GZFile} ${P2}
retval=$?
if [ ${retval} -ne 0 ]; then
if [ -n "${LOCKFILE}" ]; then
rm -f ${LOCKFILE}
fi
exit ${retval}
fi
PROGRESS=0
done
if [ -n "${LOCKFILE}" ]; then
rm -f ${LOCKFILE}
fi
exit
